from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class SpecificEntropyUnits(Enum):
        """
            SpecificEntropyUnits enumeration
        """
        
        JoulePerKilogramKelvin = 'JoulePerKilogramKelvin'
        """
            
        """
        
        JoulePerKilogramDegreeCelsius = 'JoulePerKilogramDegreeCelsius'
        """
            
        """
        
        CaloriePerGramKelvin = 'CaloriePerGramKelvin'
        """
            
        """
        
        BtuPerPoundFahrenheit = 'BtuPerPoundFahrenheit'
        """
            
        """
        
        KilojoulePerKilogramKelvin = 'KilojoulePerKilogramKelvin'
        """
            
        """
        
        MegajoulePerKilogramKelvin = 'MegajoulePerKilogramKelvin'
        """
            
        """
        
        KilojoulePerKilogramDegreeCelsius = 'KilojoulePerKilogramDegreeCelsius'
        """
            
        """
        
        MegajoulePerKilogramDegreeCelsius = 'MegajoulePerKilogramDegreeCelsius'
        """
            
        """
        
        KilocaloriePerGramKelvin = 'KilocaloriePerGramKelvin'
        """
            
        """
        

class SpecificEntropyDto:
    """
    A DTO representation of a SpecificEntropy

    Attributes:
        value (float): The value of the SpecificEntropy.
        unit (SpecificEntropyUnits): The specific unit that the SpecificEntropy value is representing.
    """

    def __init__(self, value: float, unit: SpecificEntropyUnits):
        """
        Create a new DTO representation of a SpecificEntropy

        Parameters:
            value (float): The value of the SpecificEntropy.
            unit (SpecificEntropyUnits): The specific unit that the SpecificEntropy value is representing.
        """
        self.value: float = value
        """
        The value of the SpecificEntropy
        """
        self.unit: SpecificEntropyUnits = unit
        """
        The specific unit that the SpecificEntropy value is representing
        """

    def to_json(self):
        """
        Get a SpecificEntropy DTO JSON object representing the current unit.

        :return: JSON object represents SpecificEntropy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerKilogramKelvin"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of SpecificEntropy DTO from a json representation.

        :param data: The SpecificEntropy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerKilogramKelvin"}
        :return: A new instance of SpecificEntropyDto.
        :rtype: SpecificEntropyDto
        """
        return SpecificEntropyDto(value=data["value"], unit=SpecificEntropyUnits(data["unit"]))


class SpecificEntropy(AbstractMeasure):
    """
    Specific entropy is an amount of energy required to raise temperature of a substance by 1 Kelvin per unit mass.

    Args:
        value (float): The value.
        from_unit (SpecificEntropyUnits): The SpecificEntropy unit to create from, The default unit is JoulePerKilogramKelvin
    """
    def __init__(self, value: float, from_unit: SpecificEntropyUnits = SpecificEntropyUnits.JoulePerKilogramKelvin):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__joules_per_kilogram_kelvin = None
        
        self.__joules_per_kilogram_degree_celsius = None
        
        self.__calories_per_gram_kelvin = None
        
        self.__btus_per_pound_fahrenheit = None
        
        self.__kilojoules_per_kilogram_kelvin = None
        
        self.__megajoules_per_kilogram_kelvin = None
        
        self.__kilojoules_per_kilogram_degree_celsius = None
        
        self.__megajoules_per_kilogram_degree_celsius = None
        
        self.__kilocalories_per_gram_kelvin = None
        

    def convert(self, unit: SpecificEntropyUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: SpecificEntropyUnits = SpecificEntropyUnits.JoulePerKilogramKelvin) -> SpecificEntropyDto:
        """
        Get a new instance of SpecificEntropy DTO representing the current unit.

        :param hold_in_unit: The specific SpecificEntropy unit to store the SpecificEntropy value in the DTO representation.
        :type hold_in_unit: SpecificEntropyUnits
        :return: A new instance of SpecificEntropyDto.
        :rtype: SpecificEntropyDto
        """
        return SpecificEntropyDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: SpecificEntropyUnits = SpecificEntropyUnits.JoulePerKilogramKelvin):
        """
        Get a SpecificEntropy DTO JSON object representing the current unit.

        :param hold_in_unit: The specific SpecificEntropy unit to store the SpecificEntropy value in the DTO representation.
        :type hold_in_unit: SpecificEntropyUnits
        :return: JSON object represents SpecificEntropy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerKilogramKelvin"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(specific_entropy_dto: SpecificEntropyDto):
        """
        Obtain a new instance of SpecificEntropy from a DTO unit object.

        :param specific_entropy_dto: The SpecificEntropy DTO representation.
        :type specific_entropy_dto: SpecificEntropyDto
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(specific_entropy_dto.value, specific_entropy_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of SpecificEntropy from a DTO unit json representation.

        :param data: The SpecificEntropy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerKilogramKelvin"}
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy.from_dto(SpecificEntropyDto.from_json(data))

    def __convert_from_base(self, from_unit: SpecificEntropyUnits) -> float:
        value = self._value
        
        if from_unit == SpecificEntropyUnits.JoulePerKilogramKelvin:
            return (value)
        
        if from_unit == SpecificEntropyUnits.JoulePerKilogramDegreeCelsius:
            return (value)
        
        if from_unit == SpecificEntropyUnits.CaloriePerGramKelvin:
            return (value / 4.184e3)
        
        if from_unit == SpecificEntropyUnits.BtuPerPoundFahrenheit:
            return (value / 4.1868e3)
        
        if from_unit == SpecificEntropyUnits.KilojoulePerKilogramKelvin:
            return ((value) / 1000.0)
        
        if from_unit == SpecificEntropyUnits.MegajoulePerKilogramKelvin:
            return ((value) / 1000000.0)
        
        if from_unit == SpecificEntropyUnits.KilojoulePerKilogramDegreeCelsius:
            return ((value) / 1000.0)
        
        if from_unit == SpecificEntropyUnits.MegajoulePerKilogramDegreeCelsius:
            return ((value) / 1000000.0)
        
        if from_unit == SpecificEntropyUnits.KilocaloriePerGramKelvin:
            return ((value / 4.184e3) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: SpecificEntropyUnits) -> float:
        
        if to_unit == SpecificEntropyUnits.JoulePerKilogramKelvin:
            return (value)
        
        if to_unit == SpecificEntropyUnits.JoulePerKilogramDegreeCelsius:
            return (value)
        
        if to_unit == SpecificEntropyUnits.CaloriePerGramKelvin:
            return (value * 4.184e3)
        
        if to_unit == SpecificEntropyUnits.BtuPerPoundFahrenheit:
            return (value * 4.1868e3)
        
        if to_unit == SpecificEntropyUnits.KilojoulePerKilogramKelvin:
            return ((value) * 1000.0)
        
        if to_unit == SpecificEntropyUnits.MegajoulePerKilogramKelvin:
            return ((value) * 1000000.0)
        
        if to_unit == SpecificEntropyUnits.KilojoulePerKilogramDegreeCelsius:
            return ((value) * 1000.0)
        
        if to_unit == SpecificEntropyUnits.MegajoulePerKilogramDegreeCelsius:
            return ((value) * 1000000.0)
        
        if to_unit == SpecificEntropyUnits.KilocaloriePerGramKelvin:
            return ((value * 4.184e3) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_joules_per_kilogram_kelvin(joules_per_kilogram_kelvin: float):
        """
        Create a new instance of SpecificEntropy from a value in joules_per_kilogram_kelvin.

        

        :param meters: The SpecificEntropy value in joules_per_kilogram_kelvin.
        :type joules_per_kilogram_kelvin: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(joules_per_kilogram_kelvin, SpecificEntropyUnits.JoulePerKilogramKelvin)

    
    @staticmethod
    def from_joules_per_kilogram_degree_celsius(joules_per_kilogram_degree_celsius: float):
        """
        Create a new instance of SpecificEntropy from a value in joules_per_kilogram_degree_celsius.

        

        :param meters: The SpecificEntropy value in joules_per_kilogram_degree_celsius.
        :type joules_per_kilogram_degree_celsius: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(joules_per_kilogram_degree_celsius, SpecificEntropyUnits.JoulePerKilogramDegreeCelsius)

    
    @staticmethod
    def from_calories_per_gram_kelvin(calories_per_gram_kelvin: float):
        """
        Create a new instance of SpecificEntropy from a value in calories_per_gram_kelvin.

        

        :param meters: The SpecificEntropy value in calories_per_gram_kelvin.
        :type calories_per_gram_kelvin: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(calories_per_gram_kelvin, SpecificEntropyUnits.CaloriePerGramKelvin)

    
    @staticmethod
    def from_btus_per_pound_fahrenheit(btus_per_pound_fahrenheit: float):
        """
        Create a new instance of SpecificEntropy from a value in btus_per_pound_fahrenheit.

        

        :param meters: The SpecificEntropy value in btus_per_pound_fahrenheit.
        :type btus_per_pound_fahrenheit: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(btus_per_pound_fahrenheit, SpecificEntropyUnits.BtuPerPoundFahrenheit)

    
    @staticmethod
    def from_kilojoules_per_kilogram_kelvin(kilojoules_per_kilogram_kelvin: float):
        """
        Create a new instance of SpecificEntropy from a value in kilojoules_per_kilogram_kelvin.

        

        :param meters: The SpecificEntropy value in kilojoules_per_kilogram_kelvin.
        :type kilojoules_per_kilogram_kelvin: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(kilojoules_per_kilogram_kelvin, SpecificEntropyUnits.KilojoulePerKilogramKelvin)

    
    @staticmethod
    def from_megajoules_per_kilogram_kelvin(megajoules_per_kilogram_kelvin: float):
        """
        Create a new instance of SpecificEntropy from a value in megajoules_per_kilogram_kelvin.

        

        :param meters: The SpecificEntropy value in megajoules_per_kilogram_kelvin.
        :type megajoules_per_kilogram_kelvin: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(megajoules_per_kilogram_kelvin, SpecificEntropyUnits.MegajoulePerKilogramKelvin)

    
    @staticmethod
    def from_kilojoules_per_kilogram_degree_celsius(kilojoules_per_kilogram_degree_celsius: float):
        """
        Create a new instance of SpecificEntropy from a value in kilojoules_per_kilogram_degree_celsius.

        

        :param meters: The SpecificEntropy value in kilojoules_per_kilogram_degree_celsius.
        :type kilojoules_per_kilogram_degree_celsius: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(kilojoules_per_kilogram_degree_celsius, SpecificEntropyUnits.KilojoulePerKilogramDegreeCelsius)

    
    @staticmethod
    def from_megajoules_per_kilogram_degree_celsius(megajoules_per_kilogram_degree_celsius: float):
        """
        Create a new instance of SpecificEntropy from a value in megajoules_per_kilogram_degree_celsius.

        

        :param meters: The SpecificEntropy value in megajoules_per_kilogram_degree_celsius.
        :type megajoules_per_kilogram_degree_celsius: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(megajoules_per_kilogram_degree_celsius, SpecificEntropyUnits.MegajoulePerKilogramDegreeCelsius)

    
    @staticmethod
    def from_kilocalories_per_gram_kelvin(kilocalories_per_gram_kelvin: float):
        """
        Create a new instance of SpecificEntropy from a value in kilocalories_per_gram_kelvin.

        

        :param meters: The SpecificEntropy value in kilocalories_per_gram_kelvin.
        :type kilocalories_per_gram_kelvin: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(kilocalories_per_gram_kelvin, SpecificEntropyUnits.KilocaloriePerGramKelvin)

    
    @property
    def joules_per_kilogram_kelvin(self) -> float:
        """
        
        """
        if self.__joules_per_kilogram_kelvin != None:
            return self.__joules_per_kilogram_kelvin
        self.__joules_per_kilogram_kelvin = self.__convert_from_base(SpecificEntropyUnits.JoulePerKilogramKelvin)
        return self.__joules_per_kilogram_kelvin

    
    @property
    def joules_per_kilogram_degree_celsius(self) -> float:
        """
        
        """
        if self.__joules_per_kilogram_degree_celsius != None:
            return self.__joules_per_kilogram_degree_celsius
        self.__joules_per_kilogram_degree_celsius = self.__convert_from_base(SpecificEntropyUnits.JoulePerKilogramDegreeCelsius)
        return self.__joules_per_kilogram_degree_celsius

    
    @property
    def calories_per_gram_kelvin(self) -> float:
        """
        
        """
        if self.__calories_per_gram_kelvin != None:
            return self.__calories_per_gram_kelvin
        self.__calories_per_gram_kelvin = self.__convert_from_base(SpecificEntropyUnits.CaloriePerGramKelvin)
        return self.__calories_per_gram_kelvin

    
    @property
    def btus_per_pound_fahrenheit(self) -> float:
        """
        
        """
        if self.__btus_per_pound_fahrenheit != None:
            return self.__btus_per_pound_fahrenheit
        self.__btus_per_pound_fahrenheit = self.__convert_from_base(SpecificEntropyUnits.BtuPerPoundFahrenheit)
        return self.__btus_per_pound_fahrenheit

    
    @property
    def kilojoules_per_kilogram_kelvin(self) -> float:
        """
        
        """
        if self.__kilojoules_per_kilogram_kelvin != None:
            return self.__kilojoules_per_kilogram_kelvin
        self.__kilojoules_per_kilogram_kelvin = self.__convert_from_base(SpecificEntropyUnits.KilojoulePerKilogramKelvin)
        return self.__kilojoules_per_kilogram_kelvin

    
    @property
    def megajoules_per_kilogram_kelvin(self) -> float:
        """
        
        """
        if self.__megajoules_per_kilogram_kelvin != None:
            return self.__megajoules_per_kilogram_kelvin
        self.__megajoules_per_kilogram_kelvin = self.__convert_from_base(SpecificEntropyUnits.MegajoulePerKilogramKelvin)
        return self.__megajoules_per_kilogram_kelvin

    
    @property
    def kilojoules_per_kilogram_degree_celsius(self) -> float:
        """
        
        """
        if self.__kilojoules_per_kilogram_degree_celsius != None:
            return self.__kilojoules_per_kilogram_degree_celsius
        self.__kilojoules_per_kilogram_degree_celsius = self.__convert_from_base(SpecificEntropyUnits.KilojoulePerKilogramDegreeCelsius)
        return self.__kilojoules_per_kilogram_degree_celsius

    
    @property
    def megajoules_per_kilogram_degree_celsius(self) -> float:
        """
        
        """
        if self.__megajoules_per_kilogram_degree_celsius != None:
            return self.__megajoules_per_kilogram_degree_celsius
        self.__megajoules_per_kilogram_degree_celsius = self.__convert_from_base(SpecificEntropyUnits.MegajoulePerKilogramDegreeCelsius)
        return self.__megajoules_per_kilogram_degree_celsius

    
    @property
    def kilocalories_per_gram_kelvin(self) -> float:
        """
        
        """
        if self.__kilocalories_per_gram_kelvin != None:
            return self.__kilocalories_per_gram_kelvin
        self.__kilocalories_per_gram_kelvin = self.__convert_from_base(SpecificEntropyUnits.KilocaloriePerGramKelvin)
        return self.__kilocalories_per_gram_kelvin

    
    def to_string(self, unit: SpecificEntropyUnits = SpecificEntropyUnits.JoulePerKilogramKelvin, fractional_digits: int = None) -> str:
        """
        Format the SpecificEntropy to a string.
        
        Note: the default format for SpecificEntropy is JoulePerKilogramKelvin.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the SpecificEntropy. Default is 'JoulePerKilogramKelvin'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == SpecificEntropyUnits.JoulePerKilogramKelvin:
            return f"""{super()._truncate_fraction_digits(self.joules_per_kilogram_kelvin, fractional_digits)} J/kg.K"""
        
        if unit == SpecificEntropyUnits.JoulePerKilogramDegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.joules_per_kilogram_degree_celsius, fractional_digits)} J/kg.C"""
        
        if unit == SpecificEntropyUnits.CaloriePerGramKelvin:
            return f"""{super()._truncate_fraction_digits(self.calories_per_gram_kelvin, fractional_digits)} cal/g.K"""
        
        if unit == SpecificEntropyUnits.BtuPerPoundFahrenheit:
            return f"""{super()._truncate_fraction_digits(self.btus_per_pound_fahrenheit, fractional_digits)} BTU/lb·°F"""
        
        if unit == SpecificEntropyUnits.KilojoulePerKilogramKelvin:
            return f"""{super()._truncate_fraction_digits(self.kilojoules_per_kilogram_kelvin, fractional_digits)} kJ/kg.K"""
        
        if unit == SpecificEntropyUnits.MegajoulePerKilogramKelvin:
            return f"""{super()._truncate_fraction_digits(self.megajoules_per_kilogram_kelvin, fractional_digits)} MJ/kg.K"""
        
        if unit == SpecificEntropyUnits.KilojoulePerKilogramDegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.kilojoules_per_kilogram_degree_celsius, fractional_digits)} kJ/kg.C"""
        
        if unit == SpecificEntropyUnits.MegajoulePerKilogramDegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.megajoules_per_kilogram_degree_celsius, fractional_digits)} MJ/kg.C"""
        
        if unit == SpecificEntropyUnits.KilocaloriePerGramKelvin:
            return f"""{super()._truncate_fraction_digits(self.kilocalories_per_gram_kelvin, fractional_digits)} kcal/g.K"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: SpecificEntropyUnits = SpecificEntropyUnits.JoulePerKilogramKelvin) -> str:
        """
        Get SpecificEntropy unit abbreviation.
        Note! the default abbreviation for SpecificEntropy is JoulePerKilogramKelvin.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == SpecificEntropyUnits.JoulePerKilogramKelvin:
            return """J/kg.K"""
        
        if unit_abbreviation == SpecificEntropyUnits.JoulePerKilogramDegreeCelsius:
            return """J/kg.C"""
        
        if unit_abbreviation == SpecificEntropyUnits.CaloriePerGramKelvin:
            return """cal/g.K"""
        
        if unit_abbreviation == SpecificEntropyUnits.BtuPerPoundFahrenheit:
            return """BTU/lb·°F"""
        
        if unit_abbreviation == SpecificEntropyUnits.KilojoulePerKilogramKelvin:
            return """kJ/kg.K"""
        
        if unit_abbreviation == SpecificEntropyUnits.MegajoulePerKilogramKelvin:
            return """MJ/kg.K"""
        
        if unit_abbreviation == SpecificEntropyUnits.KilojoulePerKilogramDegreeCelsius:
            return """kJ/kg.C"""
        
        if unit_abbreviation == SpecificEntropyUnits.MegajoulePerKilogramDegreeCelsius:
            return """MJ/kg.C"""
        
        if unit_abbreviation == SpecificEntropyUnits.KilocaloriePerGramKelvin:
            return """kcal/g.K"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class RatioChangeRateUnits(Enum):
        """
            RatioChangeRateUnits enumeration
        """
        
        PercentPerSecond = 'PercentPerSecond'
        """
            
        """
        
        DecimalFractionPerSecond = 'DecimalFractionPerSecond'
        """
            
        """
        

class RatioChangeRateDto:
    """
    A DTO representation of a RatioChangeRate

    Attributes:
        value (float): The value of the RatioChangeRate.
        unit (RatioChangeRateUnits): The specific unit that the RatioChangeRate value is representing.
    """

    def __init__(self, value: float, unit: RatioChangeRateUnits):
        """
        Create a new DTO representation of a RatioChangeRate

        Parameters:
            value (float): The value of the RatioChangeRate.
            unit (RatioChangeRateUnits): The specific unit that the RatioChangeRate value is representing.
        """
        self.value: float = value
        """
        The value of the RatioChangeRate
        """
        self.unit: RatioChangeRateUnits = unit
        """
        The specific unit that the RatioChangeRate value is representing
        """

    def to_json(self):
        """
        Get a RatioChangeRate DTO JSON object representing the current unit.

        :return: JSON object represents RatioChangeRate DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "DecimalFractionPerSecond"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of RatioChangeRate DTO from a json representation.

        :param data: The RatioChangeRate DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "DecimalFractionPerSecond"}
        :return: A new instance of RatioChangeRateDto.
        :rtype: RatioChangeRateDto
        """
        return RatioChangeRateDto(value=data["value"], unit=RatioChangeRateUnits(data["unit"]))


class RatioChangeRate(AbstractMeasure):
    """
    The change in ratio per unit of time.

    Args:
        value (float): The value.
        from_unit (RatioChangeRateUnits): The RatioChangeRate unit to create from, The default unit is DecimalFractionPerSecond
    """
    def __init__(self, value: float, from_unit: RatioChangeRateUnits = RatioChangeRateUnits.DecimalFractionPerSecond):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__percents_per_second = None
        
        self.__decimal_fractions_per_second = None
        

    def convert(self, unit: RatioChangeRateUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: RatioChangeRateUnits = RatioChangeRateUnits.DecimalFractionPerSecond) -> RatioChangeRateDto:
        """
        Get a new instance of RatioChangeRate DTO representing the current unit.

        :param hold_in_unit: The specific RatioChangeRate unit to store the RatioChangeRate value in the DTO representation.
        :type hold_in_unit: RatioChangeRateUnits
        :return: A new instance of RatioChangeRateDto.
        :rtype: RatioChangeRateDto
        """
        return RatioChangeRateDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: RatioChangeRateUnits = RatioChangeRateUnits.DecimalFractionPerSecond):
        """
        Get a RatioChangeRate DTO JSON object representing the current unit.

        :param hold_in_unit: The specific RatioChangeRate unit to store the RatioChangeRate value in the DTO representation.
        :type hold_in_unit: RatioChangeRateUnits
        :return: JSON object represents RatioChangeRate DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "DecimalFractionPerSecond"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(ratio_change_rate_dto: RatioChangeRateDto):
        """
        Obtain a new instance of RatioChangeRate from a DTO unit object.

        :param ratio_change_rate_dto: The RatioChangeRate DTO representation.
        :type ratio_change_rate_dto: RatioChangeRateDto
        :return: A new instance of RatioChangeRate.
        :rtype: RatioChangeRate
        """
        return RatioChangeRate(ratio_change_rate_dto.value, ratio_change_rate_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of RatioChangeRate from a DTO unit json representation.

        :param data: The RatioChangeRate DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "DecimalFractionPerSecond"}
        :return: A new instance of RatioChangeRate.
        :rtype: RatioChangeRate
        """
        return RatioChangeRate.from_dto(RatioChangeRateDto.from_json(data))

    def __convert_from_base(self, from_unit: RatioChangeRateUnits) -> float:
        value = self._value
        
        if from_unit == RatioChangeRateUnits.PercentPerSecond:
            return (value * 1e2)
        
        if from_unit == RatioChangeRateUnits.DecimalFractionPerSecond:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: RatioChangeRateUnits) -> float:
        
        if to_unit == RatioChangeRateUnits.PercentPerSecond:
            return (value / 1e2)
        
        if to_unit == RatioChangeRateUnits.DecimalFractionPerSecond:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_percents_per_second(percents_per_second: float):
        """
        Create a new instance of RatioChangeRate from a value in percents_per_second.

        

        :param meters: The RatioChangeRate value in percents_per_second.
        :type percents_per_second: float
        :return: A new instance of RatioChangeRate.
        :rtype: RatioChangeRate
        """
        return RatioChangeRate(percents_per_second, RatioChangeRateUnits.PercentPerSecond)

    
    @staticmethod
    def from_decimal_fractions_per_second(decimal_fractions_per_second: float):
        """
        Create a new instance of RatioChangeRate from a value in decimal_fractions_per_second.

        

        :param meters: The RatioChangeRate value in decimal_fractions_per_second.
        :type decimal_fractions_per_second: float
        :return: A new instance of RatioChangeRate.
        :rtype: RatioChangeRate
        """
        return RatioChangeRate(decimal_fractions_per_second, RatioChangeRateUnits.DecimalFractionPerSecond)

    
    @property
    def percents_per_second(self) -> float:
        """
        
        """
        if self.__percents_per_second != None:
            return self.__percents_per_second
        self.__percents_per_second = self.__convert_from_base(RatioChangeRateUnits.PercentPerSecond)
        return self.__percents_per_second

    
    @property
    def decimal_fractions_per_second(self) -> float:
        """
        
        """
        if self.__decimal_fractions_per_second != None:
            return self.__decimal_fractions_per_second
        self.__decimal_fractions_per_second = self.__convert_from_base(RatioChangeRateUnits.DecimalFractionPerSecond)
        return self.__decimal_fractions_per_second

    
    def to_string(self, unit: RatioChangeRateUnits = RatioChangeRateUnits.DecimalFractionPerSecond, fractional_digits: int = None) -> str:
        """
        Format the RatioChangeRate to a string.
        
        Note: the default format for RatioChangeRate is DecimalFractionPerSecond.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the RatioChangeRate. Default is 'DecimalFractionPerSecond'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == RatioChangeRateUnits.PercentPerSecond:
            return f"""{super()._truncate_fraction_digits(self.percents_per_second, fractional_digits)} %/s"""
        
        if unit == RatioChangeRateUnits.DecimalFractionPerSecond:
            return f"""{super()._truncate_fraction_digits(self.decimal_fractions_per_second, fractional_digits)} /s"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: RatioChangeRateUnits = RatioChangeRateUnits.DecimalFractionPerSecond) -> str:
        """
        Get RatioChangeRate unit abbreviation.
        Note! the default abbreviation for RatioChangeRate is DecimalFractionPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == RatioChangeRateUnits.PercentPerSecond:
            return """%/s"""
        
        if unit_abbreviation == RatioChangeRateUnits.DecimalFractionPerSecond:
            return """/s"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class KinematicViscosityUnits(Enum):
        """
            KinematicViscosityUnits enumeration
        """
        
        SquareMeterPerSecond = 'SquareMeterPerSecond'
        """
            
        """
        
        Stokes = 'Stokes'
        """
            
        """
        
        SquareFootPerSecond = 'SquareFootPerSecond'
        """
            
        """
        
        Nanostokes = 'Nanostokes'
        """
            
        """
        
        Microstokes = 'Microstokes'
        """
            
        """
        
        Millistokes = 'Millistokes'
        """
            
        """
        
        Centistokes = 'Centistokes'
        """
            
        """
        
        Decistokes = 'Decistokes'
        """
            
        """
        
        Kilostokes = 'Kilostokes'
        """
            
        """
        

class KinematicViscosityDto:
    """
    A DTO representation of a KinematicViscosity

    Attributes:
        value (float): The value of the KinematicViscosity.
        unit (KinematicViscosityUnits): The specific unit that the KinematicViscosity value is representing.
    """

    def __init__(self, value: float, unit: KinematicViscosityUnits):
        """
        Create a new DTO representation of a KinematicViscosity

        Parameters:
            value (float): The value of the KinematicViscosity.
            unit (KinematicViscosityUnits): The specific unit that the KinematicViscosity value is representing.
        """
        self.value: float = value
        """
        The value of the KinematicViscosity
        """
        self.unit: KinematicViscosityUnits = unit
        """
        The specific unit that the KinematicViscosity value is representing
        """

    def to_json(self):
        """
        Get a KinematicViscosity DTO JSON object representing the current unit.

        :return: JSON object represents KinematicViscosity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "SquareMeterPerSecond"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of KinematicViscosity DTO from a json representation.

        :param data: The KinematicViscosity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "SquareMeterPerSecond"}
        :return: A new instance of KinematicViscosityDto.
        :rtype: KinematicViscosityDto
        """
        return KinematicViscosityDto(value=data["value"], unit=KinematicViscosityUnits(data["unit"]))


class KinematicViscosity(AbstractMeasure):
    """
    The viscosity of a fluid is a measure of its resistance to gradual deformation by shear stress or tensile stress.

    Args:
        value (float): The value.
        from_unit (KinematicViscosityUnits): The KinematicViscosity unit to create from, The default unit is SquareMeterPerSecond
    """
    def __init__(self, value: float, from_unit: KinematicViscosityUnits = KinematicViscosityUnits.SquareMeterPerSecond):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__square_meters_per_second = None
        
        self.__stokes = None
        
        self.__square_feet_per_second = None
        
        self.__nanostokes = None
        
        self.__microstokes = None
        
        self.__millistokes = None
        
        self.__centistokes = None
        
        self.__decistokes = None
        
        self.__kilostokes = None
        

    def convert(self, unit: KinematicViscosityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: KinematicViscosityUnits = KinematicViscosityUnits.SquareMeterPerSecond) -> KinematicViscosityDto:
        """
        Get a new instance of KinematicViscosity DTO representing the current unit.

        :param hold_in_unit: The specific KinematicViscosity unit to store the KinematicViscosity value in the DTO representation.
        :type hold_in_unit: KinematicViscosityUnits
        :return: A new instance of KinematicViscosityDto.
        :rtype: KinematicViscosityDto
        """
        return KinematicViscosityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: KinematicViscosityUnits = KinematicViscosityUnits.SquareMeterPerSecond):
        """
        Get a KinematicViscosity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific KinematicViscosity unit to store the KinematicViscosity value in the DTO representation.
        :type hold_in_unit: KinematicViscosityUnits
        :return: JSON object represents KinematicViscosity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "SquareMeterPerSecond"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(kinematic_viscosity_dto: KinematicViscosityDto):
        """
        Obtain a new instance of KinematicViscosity from a DTO unit object.

        :param kinematic_viscosity_dto: The KinematicViscosity DTO representation.
        :type kinematic_viscosity_dto: KinematicViscosityDto
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(kinematic_viscosity_dto.value, kinematic_viscosity_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of KinematicViscosity from a DTO unit json representation.

        :param data: The KinematicViscosity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "SquareMeterPerSecond"}
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity.from_dto(KinematicViscosityDto.from_json(data))

    def __convert_from_base(self, from_unit: KinematicViscosityUnits) -> float:
        value = self._value
        
        if from_unit == KinematicViscosityUnits.SquareMeterPerSecond:
            return (value)
        
        if from_unit == KinematicViscosityUnits.Stokes:
            return (value * 1e4)
        
        if from_unit == KinematicViscosityUnits.SquareFootPerSecond:
            return (value * 10.7639)
        
        if from_unit == KinematicViscosityUnits.Nanostokes:
            return ((value * 1e4) / 1e-09)
        
        if from_unit == KinematicViscosityUnits.Microstokes:
            return ((value * 1e4) / 1e-06)
        
        if from_unit == KinematicViscosityUnits.Millistokes:
            return ((value * 1e4) / 0.001)
        
        if from_unit == KinematicViscosityUnits.Centistokes:
            return ((value * 1e4) / 0.01)
        
        if from_unit == KinematicViscosityUnits.Decistokes:
            return ((value * 1e4) / 0.1)
        
        if from_unit == KinematicViscosityUnits.Kilostokes:
            return ((value * 1e4) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: KinematicViscosityUnits) -> float:
        
        if to_unit == KinematicViscosityUnits.SquareMeterPerSecond:
            return (value)
        
        if to_unit == KinematicViscosityUnits.Stokes:
            return (value / 1e4)
        
        if to_unit == KinematicViscosityUnits.SquareFootPerSecond:
            return (value / 10.7639)
        
        if to_unit == KinematicViscosityUnits.Nanostokes:
            return ((value / 1e4) * 1e-09)
        
        if to_unit == KinematicViscosityUnits.Microstokes:
            return ((value / 1e4) * 1e-06)
        
        if to_unit == KinematicViscosityUnits.Millistokes:
            return ((value / 1e4) * 0.001)
        
        if to_unit == KinematicViscosityUnits.Centistokes:
            return ((value / 1e4) * 0.01)
        
        if to_unit == KinematicViscosityUnits.Decistokes:
            return ((value / 1e4) * 0.1)
        
        if to_unit == KinematicViscosityUnits.Kilostokes:
            return ((value / 1e4) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_square_meters_per_second(square_meters_per_second: float):
        """
        Create a new instance of KinematicViscosity from a value in square_meters_per_second.

        

        :param meters: The KinematicViscosity value in square_meters_per_second.
        :type square_meters_per_second: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(square_meters_per_second, KinematicViscosityUnits.SquareMeterPerSecond)

    
    @staticmethod
    def from_stokes(stokes: float):
        """
        Create a new instance of KinematicViscosity from a value in stokes.

        

        :param meters: The KinematicViscosity value in stokes.
        :type stokes: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(stokes, KinematicViscosityUnits.Stokes)

    
    @staticmethod
    def from_square_feet_per_second(square_feet_per_second: float):
        """
        Create a new instance of KinematicViscosity from a value in square_feet_per_second.

        

        :param meters: The KinematicViscosity value in square_feet_per_second.
        :type square_feet_per_second: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(square_feet_per_second, KinematicViscosityUnits.SquareFootPerSecond)

    
    @staticmethod
    def from_nanostokes(nanostokes: float):
        """
        Create a new instance of KinematicViscosity from a value in nanostokes.

        

        :param meters: The KinematicViscosity value in nanostokes.
        :type nanostokes: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(nanostokes, KinematicViscosityUnits.Nanostokes)

    
    @staticmethod
    def from_microstokes(microstokes: float):
        """
        Create a new instance of KinematicViscosity from a value in microstokes.

        

        :param meters: The KinematicViscosity value in microstokes.
        :type microstokes: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(microstokes, KinematicViscosityUnits.Microstokes)

    
    @staticmethod
    def from_millistokes(millistokes: float):
        """
        Create a new instance of KinematicViscosity from a value in millistokes.

        

        :param meters: The KinematicViscosity value in millistokes.
        :type millistokes: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(millistokes, KinematicViscosityUnits.Millistokes)

    
    @staticmethod
    def from_centistokes(centistokes: float):
        """
        Create a new instance of KinematicViscosity from a value in centistokes.

        

        :param meters: The KinematicViscosity value in centistokes.
        :type centistokes: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(centistokes, KinematicViscosityUnits.Centistokes)

    
    @staticmethod
    def from_decistokes(decistokes: float):
        """
        Create a new instance of KinematicViscosity from a value in decistokes.

        

        :param meters: The KinematicViscosity value in decistokes.
        :type decistokes: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(decistokes, KinematicViscosityUnits.Decistokes)

    
    @staticmethod
    def from_kilostokes(kilostokes: float):
        """
        Create a new instance of KinematicViscosity from a value in kilostokes.

        

        :param meters: The KinematicViscosity value in kilostokes.
        :type kilostokes: float
        :return: A new instance of KinematicViscosity.
        :rtype: KinematicViscosity
        """
        return KinematicViscosity(kilostokes, KinematicViscosityUnits.Kilostokes)

    
    @property
    def square_meters_per_second(self) -> float:
        """
        
        """
        if self.__square_meters_per_second != None:
            return self.__square_meters_per_second
        self.__square_meters_per_second = self.__convert_from_base(KinematicViscosityUnits.SquareMeterPerSecond)
        return self.__square_meters_per_second

    
    @property
    def stokes(self) -> float:
        """
        
        """
        if self.__stokes != None:
            return self.__stokes
        self.__stokes = self.__convert_from_base(KinematicViscosityUnits.Stokes)
        return self.__stokes

    
    @property
    def square_feet_per_second(self) -> float:
        """
        
        """
        if self.__square_feet_per_second != None:
            return self.__square_feet_per_second
        self.__square_feet_per_second = self.__convert_from_base(KinematicViscosityUnits.SquareFootPerSecond)
        return self.__square_feet_per_second

    
    @property
    def nanostokes(self) -> float:
        """
        
        """
        if self.__nanostokes != None:
            return self.__nanostokes
        self.__nanostokes = self.__convert_from_base(KinematicViscosityUnits.Nanostokes)
        return self.__nanostokes

    
    @property
    def microstokes(self) -> float:
        """
        
        """
        if self.__microstokes != None:
            return self.__microstokes
        self.__microstokes = self.__convert_from_base(KinematicViscosityUnits.Microstokes)
        return self.__microstokes

    
    @property
    def millistokes(self) -> float:
        """
        
        """
        if self.__millistokes != None:
            return self.__millistokes
        self.__millistokes = self.__convert_from_base(KinematicViscosityUnits.Millistokes)
        return self.__millistokes

    
    @property
    def centistokes(self) -> float:
        """
        
        """
        if self.__centistokes != None:
            return self.__centistokes
        self.__centistokes = self.__convert_from_base(KinematicViscosityUnits.Centistokes)
        return self.__centistokes

    
    @property
    def decistokes(self) -> float:
        """
        
        """
        if self.__decistokes != None:
            return self.__decistokes
        self.__decistokes = self.__convert_from_base(KinematicViscosityUnits.Decistokes)
        return self.__decistokes

    
    @property
    def kilostokes(self) -> float:
        """
        
        """
        if self.__kilostokes != None:
            return self.__kilostokes
        self.__kilostokes = self.__convert_from_base(KinematicViscosityUnits.Kilostokes)
        return self.__kilostokes

    
    def to_string(self, unit: KinematicViscosityUnits = KinematicViscosityUnits.SquareMeterPerSecond, fractional_digits: int = None) -> str:
        """
        Format the KinematicViscosity to a string.
        
        Note: the default format for KinematicViscosity is SquareMeterPerSecond.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the KinematicViscosity. Default is 'SquareMeterPerSecond'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == KinematicViscosityUnits.SquareMeterPerSecond:
            return f"""{super()._truncate_fraction_digits(self.square_meters_per_second, fractional_digits)} m²/s"""
        
        if unit == KinematicViscosityUnits.Stokes:
            return f"""{super()._truncate_fraction_digits(self.stokes, fractional_digits)} St"""
        
        if unit == KinematicViscosityUnits.SquareFootPerSecond:
            return f"""{super()._truncate_fraction_digits(self.square_feet_per_second, fractional_digits)} ft²/s"""
        
        if unit == KinematicViscosityUnits.Nanostokes:
            return f"""{super()._truncate_fraction_digits(self.nanostokes, fractional_digits)} nSt"""
        
        if unit == KinematicViscosityUnits.Microstokes:
            return f"""{super()._truncate_fraction_digits(self.microstokes, fractional_digits)} μSt"""
        
        if unit == KinematicViscosityUnits.Millistokes:
            return f"""{super()._truncate_fraction_digits(self.millistokes, fractional_digits)} mSt"""
        
        if unit == KinematicViscosityUnits.Centistokes:
            return f"""{super()._truncate_fraction_digits(self.centistokes, fractional_digits)} cSt"""
        
        if unit == KinematicViscosityUnits.Decistokes:
            return f"""{super()._truncate_fraction_digits(self.decistokes, fractional_digits)} dSt"""
        
        if unit == KinematicViscosityUnits.Kilostokes:
            return f"""{super()._truncate_fraction_digits(self.kilostokes, fractional_digits)} kSt"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: KinematicViscosityUnits = KinematicViscosityUnits.SquareMeterPerSecond) -> str:
        """
        Get KinematicViscosity unit abbreviation.
        Note! the default abbreviation for KinematicViscosity is SquareMeterPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == KinematicViscosityUnits.SquareMeterPerSecond:
            return """m²/s"""
        
        if unit_abbreviation == KinematicViscosityUnits.Stokes:
            return """St"""
        
        if unit_abbreviation == KinematicViscosityUnits.SquareFootPerSecond:
            return """ft²/s"""
        
        if unit_abbreviation == KinematicViscosityUnits.Nanostokes:
            return """nSt"""
        
        if unit_abbreviation == KinematicViscosityUnits.Microstokes:
            return """μSt"""
        
        if unit_abbreviation == KinematicViscosityUnits.Millistokes:
            return """mSt"""
        
        if unit_abbreviation == KinematicViscosityUnits.Centistokes:
            return """cSt"""
        
        if unit_abbreviation == KinematicViscosityUnits.Decistokes:
            return """dSt"""
        
        if unit_abbreviation == KinematicViscosityUnits.Kilostokes:
            return """kSt"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class IrradianceUnits(Enum):
        """
            IrradianceUnits enumeration
        """
        
        WattPerSquareMeter = 'WattPerSquareMeter'
        """
            
        """
        
        WattPerSquareCentimeter = 'WattPerSquareCentimeter'
        """
            
        """
        
        PicowattPerSquareMeter = 'PicowattPerSquareMeter'
        """
            
        """
        
        NanowattPerSquareMeter = 'NanowattPerSquareMeter'
        """
            
        """
        
        MicrowattPerSquareMeter = 'MicrowattPerSquareMeter'
        """
            
        """
        
        MilliwattPerSquareMeter = 'MilliwattPerSquareMeter'
        """
            
        """
        
        KilowattPerSquareMeter = 'KilowattPerSquareMeter'
        """
            
        """
        
        MegawattPerSquareMeter = 'MegawattPerSquareMeter'
        """
            
        """
        
        PicowattPerSquareCentimeter = 'PicowattPerSquareCentimeter'
        """
            
        """
        
        NanowattPerSquareCentimeter = 'NanowattPerSquareCentimeter'
        """
            
        """
        
        MicrowattPerSquareCentimeter = 'MicrowattPerSquareCentimeter'
        """
            
        """
        
        MilliwattPerSquareCentimeter = 'MilliwattPerSquareCentimeter'
        """
            
        """
        
        KilowattPerSquareCentimeter = 'KilowattPerSquareCentimeter'
        """
            
        """
        
        MegawattPerSquareCentimeter = 'MegawattPerSquareCentimeter'
        """
            
        """
        

class IrradianceDto:
    """
    A DTO representation of a Irradiance

    Attributes:
        value (float): The value of the Irradiance.
        unit (IrradianceUnits): The specific unit that the Irradiance value is representing.
    """

    def __init__(self, value: float, unit: IrradianceUnits):
        """
        Create a new DTO representation of a Irradiance

        Parameters:
            value (float): The value of the Irradiance.
            unit (IrradianceUnits): The specific unit that the Irradiance value is representing.
        """
        self.value: float = value
        """
        The value of the Irradiance
        """
        self.unit: IrradianceUnits = unit
        """
        The specific unit that the Irradiance value is representing
        """

    def to_json(self):
        """
        Get a Irradiance DTO JSON object representing the current unit.

        :return: JSON object represents Irradiance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "WattPerSquareMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Irradiance DTO from a json representation.

        :param data: The Irradiance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "WattPerSquareMeter"}
        :return: A new instance of IrradianceDto.
        :rtype: IrradianceDto
        """
        return IrradianceDto(value=data["value"], unit=IrradianceUnits(data["unit"]))


class Irradiance(AbstractMeasure):
    """
    Irradiance is the intensity of ultraviolet (UV) or visible light incident on a surface.

    Args:
        value (float): The value.
        from_unit (IrradianceUnits): The Irradiance unit to create from, The default unit is WattPerSquareMeter
    """
    def __init__(self, value: float, from_unit: IrradianceUnits = IrradianceUnits.WattPerSquareMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__watts_per_square_meter = None
        
        self.__watts_per_square_centimeter = None
        
        self.__picowatts_per_square_meter = None
        
        self.__nanowatts_per_square_meter = None
        
        self.__microwatts_per_square_meter = None
        
        self.__milliwatts_per_square_meter = None
        
        self.__kilowatts_per_square_meter = None
        
        self.__megawatts_per_square_meter = None
        
        self.__picowatts_per_square_centimeter = None
        
        self.__nanowatts_per_square_centimeter = None
        
        self.__microwatts_per_square_centimeter = None
        
        self.__milliwatts_per_square_centimeter = None
        
        self.__kilowatts_per_square_centimeter = None
        
        self.__megawatts_per_square_centimeter = None
        

    def convert(self, unit: IrradianceUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: IrradianceUnits = IrradianceUnits.WattPerSquareMeter) -> IrradianceDto:
        """
        Get a new instance of Irradiance DTO representing the current unit.

        :param hold_in_unit: The specific Irradiance unit to store the Irradiance value in the DTO representation.
        :type hold_in_unit: IrradianceUnits
        :return: A new instance of IrradianceDto.
        :rtype: IrradianceDto
        """
        return IrradianceDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: IrradianceUnits = IrradianceUnits.WattPerSquareMeter):
        """
        Get a Irradiance DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Irradiance unit to store the Irradiance value in the DTO representation.
        :type hold_in_unit: IrradianceUnits
        :return: JSON object represents Irradiance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "WattPerSquareMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(irradiance_dto: IrradianceDto):
        """
        Obtain a new instance of Irradiance from a DTO unit object.

        :param irradiance_dto: The Irradiance DTO representation.
        :type irradiance_dto: IrradianceDto
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(irradiance_dto.value, irradiance_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Irradiance from a DTO unit json representation.

        :param data: The Irradiance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "WattPerSquareMeter"}
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance.from_dto(IrradianceDto.from_json(data))

    def __convert_from_base(self, from_unit: IrradianceUnits) -> float:
        value = self._value
        
        if from_unit == IrradianceUnits.WattPerSquareMeter:
            return (value)
        
        if from_unit == IrradianceUnits.WattPerSquareCentimeter:
            return (value * 0.0001)
        
        if from_unit == IrradianceUnits.PicowattPerSquareMeter:
            return ((value) / 1e-12)
        
        if from_unit == IrradianceUnits.NanowattPerSquareMeter:
            return ((value) / 1e-09)
        
        if from_unit == IrradianceUnits.MicrowattPerSquareMeter:
            return ((value) / 1e-06)
        
        if from_unit == IrradianceUnits.MilliwattPerSquareMeter:
            return ((value) / 0.001)
        
        if from_unit == IrradianceUnits.KilowattPerSquareMeter:
            return ((value) / 1000.0)
        
        if from_unit == IrradianceUnits.MegawattPerSquareMeter:
            return ((value) / 1000000.0)
        
        if from_unit == IrradianceUnits.PicowattPerSquareCentimeter:
            return ((value * 0.0001) / 1e-12)
        
        if from_unit == IrradianceUnits.NanowattPerSquareCentimeter:
            return ((value * 0.0001) / 1e-09)
        
        if from_unit == IrradianceUnits.MicrowattPerSquareCentimeter:
            return ((value * 0.0001) / 1e-06)
        
        if from_unit == IrradianceUnits.MilliwattPerSquareCentimeter:
            return ((value * 0.0001) / 0.001)
        
        if from_unit == IrradianceUnits.KilowattPerSquareCentimeter:
            return ((value * 0.0001) / 1000.0)
        
        if from_unit == IrradianceUnits.MegawattPerSquareCentimeter:
            return ((value * 0.0001) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: IrradianceUnits) -> float:
        
        if to_unit == IrradianceUnits.WattPerSquareMeter:
            return (value)
        
        if to_unit == IrradianceUnits.WattPerSquareCentimeter:
            return (value * 10000)
        
        if to_unit == IrradianceUnits.PicowattPerSquareMeter:
            return ((value) * 1e-12)
        
        if to_unit == IrradianceUnits.NanowattPerSquareMeter:
            return ((value) * 1e-09)
        
        if to_unit == IrradianceUnits.MicrowattPerSquareMeter:
            return ((value) * 1e-06)
        
        if to_unit == IrradianceUnits.MilliwattPerSquareMeter:
            return ((value) * 0.001)
        
        if to_unit == IrradianceUnits.KilowattPerSquareMeter:
            return ((value) * 1000.0)
        
        if to_unit == IrradianceUnits.MegawattPerSquareMeter:
            return ((value) * 1000000.0)
        
        if to_unit == IrradianceUnits.PicowattPerSquareCentimeter:
            return ((value * 10000) * 1e-12)
        
        if to_unit == IrradianceUnits.NanowattPerSquareCentimeter:
            return ((value * 10000) * 1e-09)
        
        if to_unit == IrradianceUnits.MicrowattPerSquareCentimeter:
            return ((value * 10000) * 1e-06)
        
        if to_unit == IrradianceUnits.MilliwattPerSquareCentimeter:
            return ((value * 10000) * 0.001)
        
        if to_unit == IrradianceUnits.KilowattPerSquareCentimeter:
            return ((value * 10000) * 1000.0)
        
        if to_unit == IrradianceUnits.MegawattPerSquareCentimeter:
            return ((value * 10000) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_watts_per_square_meter(watts_per_square_meter: float):
        """
        Create a new instance of Irradiance from a value in watts_per_square_meter.

        

        :param meters: The Irradiance value in watts_per_square_meter.
        :type watts_per_square_meter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(watts_per_square_meter, IrradianceUnits.WattPerSquareMeter)

    
    @staticmethod
    def from_watts_per_square_centimeter(watts_per_square_centimeter: float):
        """
        Create a new instance of Irradiance from a value in watts_per_square_centimeter.

        

        :param meters: The Irradiance value in watts_per_square_centimeter.
        :type watts_per_square_centimeter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(watts_per_square_centimeter, IrradianceUnits.WattPerSquareCentimeter)

    
    @staticmethod
    def from_picowatts_per_square_meter(picowatts_per_square_meter: float):
        """
        Create a new instance of Irradiance from a value in picowatts_per_square_meter.

        

        :param meters: The Irradiance value in picowatts_per_square_meter.
        :type picowatts_per_square_meter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(picowatts_per_square_meter, IrradianceUnits.PicowattPerSquareMeter)

    
    @staticmethod
    def from_nanowatts_per_square_meter(nanowatts_per_square_meter: float):
        """
        Create a new instance of Irradiance from a value in nanowatts_per_square_meter.

        

        :param meters: The Irradiance value in nanowatts_per_square_meter.
        :type nanowatts_per_square_meter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(nanowatts_per_square_meter, IrradianceUnits.NanowattPerSquareMeter)

    
    @staticmethod
    def from_microwatts_per_square_meter(microwatts_per_square_meter: float):
        """
        Create a new instance of Irradiance from a value in microwatts_per_square_meter.

        

        :param meters: The Irradiance value in microwatts_per_square_meter.
        :type microwatts_per_square_meter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(microwatts_per_square_meter, IrradianceUnits.MicrowattPerSquareMeter)

    
    @staticmethod
    def from_milliwatts_per_square_meter(milliwatts_per_square_meter: float):
        """
        Create a new instance of Irradiance from a value in milliwatts_per_square_meter.

        

        :param meters: The Irradiance value in milliwatts_per_square_meter.
        :type milliwatts_per_square_meter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(milliwatts_per_square_meter, IrradianceUnits.MilliwattPerSquareMeter)

    
    @staticmethod
    def from_kilowatts_per_square_meter(kilowatts_per_square_meter: float):
        """
        Create a new instance of Irradiance from a value in kilowatts_per_square_meter.

        

        :param meters: The Irradiance value in kilowatts_per_square_meter.
        :type kilowatts_per_square_meter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(kilowatts_per_square_meter, IrradianceUnits.KilowattPerSquareMeter)

    
    @staticmethod
    def from_megawatts_per_square_meter(megawatts_per_square_meter: float):
        """
        Create a new instance of Irradiance from a value in megawatts_per_square_meter.

        

        :param meters: The Irradiance value in megawatts_per_square_meter.
        :type megawatts_per_square_meter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(megawatts_per_square_meter, IrradianceUnits.MegawattPerSquareMeter)

    
    @staticmethod
    def from_picowatts_per_square_centimeter(picowatts_per_square_centimeter: float):
        """
        Create a new instance of Irradiance from a value in picowatts_per_square_centimeter.

        

        :param meters: The Irradiance value in picowatts_per_square_centimeter.
        :type picowatts_per_square_centimeter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(picowatts_per_square_centimeter, IrradianceUnits.PicowattPerSquareCentimeter)

    
    @staticmethod
    def from_nanowatts_per_square_centimeter(nanowatts_per_square_centimeter: float):
        """
        Create a new instance of Irradiance from a value in nanowatts_per_square_centimeter.

        

        :param meters: The Irradiance value in nanowatts_per_square_centimeter.
        :type nanowatts_per_square_centimeter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(nanowatts_per_square_centimeter, IrradianceUnits.NanowattPerSquareCentimeter)

    
    @staticmethod
    def from_microwatts_per_square_centimeter(microwatts_per_square_centimeter: float):
        """
        Create a new instance of Irradiance from a value in microwatts_per_square_centimeter.

        

        :param meters: The Irradiance value in microwatts_per_square_centimeter.
        :type microwatts_per_square_centimeter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(microwatts_per_square_centimeter, IrradianceUnits.MicrowattPerSquareCentimeter)

    
    @staticmethod
    def from_milliwatts_per_square_centimeter(milliwatts_per_square_centimeter: float):
        """
        Create a new instance of Irradiance from a value in milliwatts_per_square_centimeter.

        

        :param meters: The Irradiance value in milliwatts_per_square_centimeter.
        :type milliwatts_per_square_centimeter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(milliwatts_per_square_centimeter, IrradianceUnits.MilliwattPerSquareCentimeter)

    
    @staticmethod
    def from_kilowatts_per_square_centimeter(kilowatts_per_square_centimeter: float):
        """
        Create a new instance of Irradiance from a value in kilowatts_per_square_centimeter.

        

        :param meters: The Irradiance value in kilowatts_per_square_centimeter.
        :type kilowatts_per_square_centimeter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(kilowatts_per_square_centimeter, IrradianceUnits.KilowattPerSquareCentimeter)

    
    @staticmethod
    def from_megawatts_per_square_centimeter(megawatts_per_square_centimeter: float):
        """
        Create a new instance of Irradiance from a value in megawatts_per_square_centimeter.

        

        :param meters: The Irradiance value in megawatts_per_square_centimeter.
        :type megawatts_per_square_centimeter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(megawatts_per_square_centimeter, IrradianceUnits.MegawattPerSquareCentimeter)

    
    @property
    def watts_per_square_meter(self) -> float:
        """
        
        """
        if self.__watts_per_square_meter != None:
            return self.__watts_per_square_meter
        self.__watts_per_square_meter = self.__convert_from_base(IrradianceUnits.WattPerSquareMeter)
        return self.__watts_per_square_meter

    
    @property
    def watts_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__watts_per_square_centimeter != None:
            return self.__watts_per_square_centimeter
        self.__watts_per_square_centimeter = self.__convert_from_base(IrradianceUnits.WattPerSquareCentimeter)
        return self.__watts_per_square_centimeter

    
    @property
    def picowatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__picowatts_per_square_meter != None:
            return self.__picowatts_per_square_meter
        self.__picowatts_per_square_meter = self.__convert_from_base(IrradianceUnits.PicowattPerSquareMeter)
        return self.__picowatts_per_square_meter

    
    @property
    def nanowatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__nanowatts_per_square_meter != None:
            return self.__nanowatts_per_square_meter
        self.__nanowatts_per_square_meter = self.__convert_from_base(IrradianceUnits.NanowattPerSquareMeter)
        return self.__nanowatts_per_square_meter

    
    @property
    def microwatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__microwatts_per_square_meter != None:
            return self.__microwatts_per_square_meter
        self.__microwatts_per_square_meter = self.__convert_from_base(IrradianceUnits.MicrowattPerSquareMeter)
        return self.__microwatts_per_square_meter

    
    @property
    def milliwatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__milliwatts_per_square_meter != None:
            return self.__milliwatts_per_square_meter
        self.__milliwatts_per_square_meter = self.__convert_from_base(IrradianceUnits.MilliwattPerSquareMeter)
        return self.__milliwatts_per_square_meter

    
    @property
    def kilowatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__kilowatts_per_square_meter != None:
            return self.__kilowatts_per_square_meter
        self.__kilowatts_per_square_meter = self.__convert_from_base(IrradianceUnits.KilowattPerSquareMeter)
        return self.__kilowatts_per_square_meter

    
    @property
    def megawatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__megawatts_per_square_meter != None:
            return self.__megawatts_per_square_meter
        self.__megawatts_per_square_meter = self.__convert_from_base(IrradianceUnits.MegawattPerSquareMeter)
        return self.__megawatts_per_square_meter

    
    @property
    def picowatts_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__picowatts_per_square_centimeter != None:
            return self.__picowatts_per_square_centimeter
        self.__picowatts_per_square_centimeter = self.__convert_from_base(IrradianceUnits.PicowattPerSquareCentimeter)
        return self.__picowatts_per_square_centimeter

    
    @property
    def nanowatts_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__nanowatts_per_square_centimeter != None:
            return self.__nanowatts_per_square_centimeter
        self.__nanowatts_per_square_centimeter = self.__convert_from_base(IrradianceUnits.NanowattPerSquareCentimeter)
        return self.__nanowatts_per_square_centimeter

    
    @property
    def microwatts_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__microwatts_per_square_centimeter != None:
            return self.__microwatts_per_square_centimeter
        self.__microwatts_per_square_centimeter = self.__convert_from_base(IrradianceUnits.MicrowattPerSquareCentimeter)
        return self.__microwatts_per_square_centimeter

    
    @property
    def milliwatts_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__milliwatts_per_square_centimeter != None:
            return self.__milliwatts_per_square_centimeter
        self.__milliwatts_per_square_centimeter = self.__convert_from_base(IrradianceUnits.MilliwattPerSquareCentimeter)
        return self.__milliwatts_per_square_centimeter

    
    @property
    def kilowatts_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__kilowatts_per_square_centimeter != None:
            return self.__kilowatts_per_square_centimeter
        self.__kilowatts_per_square_centimeter = self.__convert_from_base(IrradianceUnits.KilowattPerSquareCentimeter)
        return self.__kilowatts_per_square_centimeter

    
    @property
    def megawatts_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__megawatts_per_square_centimeter != None:
            return self.__megawatts_per_square_centimeter
        self.__megawatts_per_square_centimeter = self.__convert_from_base(IrradianceUnits.MegawattPerSquareCentimeter)
        return self.__megawatts_per_square_centimeter

    
    def to_string(self, unit: IrradianceUnits = IrradianceUnits.WattPerSquareMeter, fractional_digits: int = None) -> str:
        """
        Format the Irradiance to a string.
        
        Note: the default format for Irradiance is WattPerSquareMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Irradiance. Default is 'WattPerSquareMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == IrradianceUnits.WattPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.watts_per_square_meter, fractional_digits)} W/m²"""
        
        if unit == IrradianceUnits.WattPerSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.watts_per_square_centimeter, fractional_digits)} W/cm²"""
        
        if unit == IrradianceUnits.PicowattPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.picowatts_per_square_meter, fractional_digits)} pW/m²"""
        
        if unit == IrradianceUnits.NanowattPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.nanowatts_per_square_meter, fractional_digits)} nW/m²"""
        
        if unit == IrradianceUnits.MicrowattPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.microwatts_per_square_meter, fractional_digits)} μW/m²"""
        
        if unit == IrradianceUnits.MilliwattPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.milliwatts_per_square_meter, fractional_digits)} mW/m²"""
        
        if unit == IrradianceUnits.KilowattPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.kilowatts_per_square_meter, fractional_digits)} kW/m²"""
        
        if unit == IrradianceUnits.MegawattPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.megawatts_per_square_meter, fractional_digits)} MW/m²"""
        
        if unit == IrradianceUnits.PicowattPerSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.picowatts_per_square_centimeter, fractional_digits)} pW/cm²"""
        
        if unit == IrradianceUnits.NanowattPerSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.nanowatts_per_square_centimeter, fractional_digits)} nW/cm²"""
        
        if unit == IrradianceUnits.MicrowattPerSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.microwatts_per_square_centimeter, fractional_digits)} μW/cm²"""
        
        if unit == IrradianceUnits.MilliwattPerSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.milliwatts_per_square_centimeter, fractional_digits)} mW/cm²"""
        
        if unit == IrradianceUnits.KilowattPerSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.kilowatts_per_square_centimeter, fractional_digits)} kW/cm²"""
        
        if unit == IrradianceUnits.MegawattPerSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.megawatts_per_square_centimeter, fractional_digits)} MW/cm²"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: IrradianceUnits = IrradianceUnits.WattPerSquareMeter) -> str:
        """
        Get Irradiance unit abbreviation.
        Note! the default abbreviation for Irradiance is WattPerSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == IrradianceUnits.WattPerSquareMeter:
            return """W/m²"""
        
        if unit_abbreviation == IrradianceUnits.WattPerSquareCentimeter:
            return """W/cm²"""
        
        if unit_abbreviation == IrradianceUnits.PicowattPerSquareMeter:
            return """pW/m²"""
        
        if unit_abbreviation == IrradianceUnits.NanowattPerSquareMeter:
            return """nW/m²"""
        
        if unit_abbreviation == IrradianceUnits.MicrowattPerSquareMeter:
            return """μW/m²"""
        
        if unit_abbreviation == IrradianceUnits.MilliwattPerSquareMeter:
            return """mW/m²"""
        
        if unit_abbreviation == IrradianceUnits.KilowattPerSquareMeter:
            return """kW/m²"""
        
        if unit_abbreviation == IrradianceUnits.MegawattPerSquareMeter:
            return """MW/m²"""
        
        if unit_abbreviation == IrradianceUnits.PicowattPerSquareCentimeter:
            return """pW/cm²"""
        
        if unit_abbreviation == IrradianceUnits.NanowattPerSquareCentimeter:
            return """nW/cm²"""
        
        if unit_abbreviation == IrradianceUnits.MicrowattPerSquareCentimeter:
            return """μW/cm²"""
        
        if unit_abbreviation == IrradianceUnits.MilliwattPerSquareCentimeter:
            return """mW/cm²"""
        
        if unit_abbreviation == IrradianceUnits.KilowattPerSquareCentimeter:
            return """kW/cm²"""
        
        if unit_abbreviation == IrradianceUnits.MegawattPerSquareCentimeter:
            return """MW/cm²"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ApparentEnergyUnits(Enum):
        """
            ApparentEnergyUnits enumeration
        """
        
        VoltampereHour = 'VoltampereHour'
        """
            
        """
        
        KilovoltampereHour = 'KilovoltampereHour'
        """
            
        """
        
        MegavoltampereHour = 'MegavoltampereHour'
        """
            
        """
        

class ApparentEnergyDto:
    """
    A DTO representation of a ApparentEnergy

    Attributes:
        value (float): The value of the ApparentEnergy.
        unit (ApparentEnergyUnits): The specific unit that the ApparentEnergy value is representing.
    """

    def __init__(self, value: float, unit: ApparentEnergyUnits):
        """
        Create a new DTO representation of a ApparentEnergy

        Parameters:
            value (float): The value of the ApparentEnergy.
            unit (ApparentEnergyUnits): The specific unit that the ApparentEnergy value is representing.
        """
        self.value: float = value
        """
        The value of the ApparentEnergy
        """
        self.unit: ApparentEnergyUnits = unit
        """
        The specific unit that the ApparentEnergy value is representing
        """

    def to_json(self):
        """
        Get a ApparentEnergy DTO JSON object representing the current unit.

        :return: JSON object represents ApparentEnergy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "VoltampereHour"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ApparentEnergy DTO from a json representation.

        :param data: The ApparentEnergy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "VoltampereHour"}
        :return: A new instance of ApparentEnergyDto.
        :rtype: ApparentEnergyDto
        """
        return ApparentEnergyDto(value=data["value"], unit=ApparentEnergyUnits(data["unit"]))


class ApparentEnergy(AbstractMeasure):
    """
    A unit for expressing the integral of apparent power over time, equal to the product of 1 volt-ampere and 1 hour, or to 3600 joules.

    Args:
        value (float): The value.
        from_unit (ApparentEnergyUnits): The ApparentEnergy unit to create from, The default unit is VoltampereHour
    """
    def __init__(self, value: float, from_unit: ApparentEnergyUnits = ApparentEnergyUnits.VoltampereHour):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__voltampere_hours = None
        
        self.__kilovoltampere_hours = None
        
        self.__megavoltampere_hours = None
        

    def convert(self, unit: ApparentEnergyUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ApparentEnergyUnits = ApparentEnergyUnits.VoltampereHour) -> ApparentEnergyDto:
        """
        Get a new instance of ApparentEnergy DTO representing the current unit.

        :param hold_in_unit: The specific ApparentEnergy unit to store the ApparentEnergy value in the DTO representation.
        :type hold_in_unit: ApparentEnergyUnits
        :return: A new instance of ApparentEnergyDto.
        :rtype: ApparentEnergyDto
        """
        return ApparentEnergyDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ApparentEnergyUnits = ApparentEnergyUnits.VoltampereHour):
        """
        Get a ApparentEnergy DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ApparentEnergy unit to store the ApparentEnergy value in the DTO representation.
        :type hold_in_unit: ApparentEnergyUnits
        :return: JSON object represents ApparentEnergy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "VoltampereHour"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(apparent_energy_dto: ApparentEnergyDto):
        """
        Obtain a new instance of ApparentEnergy from a DTO unit object.

        :param apparent_energy_dto: The ApparentEnergy DTO representation.
        :type apparent_energy_dto: ApparentEnergyDto
        :return: A new instance of ApparentEnergy.
        :rtype: ApparentEnergy
        """
        return ApparentEnergy(apparent_energy_dto.value, apparent_energy_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ApparentEnergy from a DTO unit json representation.

        :param data: The ApparentEnergy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "VoltampereHour"}
        :return: A new instance of ApparentEnergy.
        :rtype: ApparentEnergy
        """
        return ApparentEnergy.from_dto(ApparentEnergyDto.from_json(data))

    def __convert_from_base(self, from_unit: ApparentEnergyUnits) -> float:
        value = self._value
        
        if from_unit == ApparentEnergyUnits.VoltampereHour:
            return (value)
        
        if from_unit == ApparentEnergyUnits.KilovoltampereHour:
            return ((value) / 1000.0)
        
        if from_unit == ApparentEnergyUnits.MegavoltampereHour:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ApparentEnergyUnits) -> float:
        
        if to_unit == ApparentEnergyUnits.VoltampereHour:
            return (value)
        
        if to_unit == ApparentEnergyUnits.KilovoltampereHour:
            return ((value) * 1000.0)
        
        if to_unit == ApparentEnergyUnits.MegavoltampereHour:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_voltampere_hours(voltampere_hours: float):
        """
        Create a new instance of ApparentEnergy from a value in voltampere_hours.

        

        :param meters: The ApparentEnergy value in voltampere_hours.
        :type voltampere_hours: float
        :return: A new instance of ApparentEnergy.
        :rtype: ApparentEnergy
        """
        return ApparentEnergy(voltampere_hours, ApparentEnergyUnits.VoltampereHour)

    
    @staticmethod
    def from_kilovoltampere_hours(kilovoltampere_hours: float):
        """
        Create a new instance of ApparentEnergy from a value in kilovoltampere_hours.

        

        :param meters: The ApparentEnergy value in kilovoltampere_hours.
        :type kilovoltampere_hours: float
        :return: A new instance of ApparentEnergy.
        :rtype: ApparentEnergy
        """
        return ApparentEnergy(kilovoltampere_hours, ApparentEnergyUnits.KilovoltampereHour)

    
    @staticmethod
    def from_megavoltampere_hours(megavoltampere_hours: float):
        """
        Create a new instance of ApparentEnergy from a value in megavoltampere_hours.

        

        :param meters: The ApparentEnergy value in megavoltampere_hours.
        :type megavoltampere_hours: float
        :return: A new instance of ApparentEnergy.
        :rtype: ApparentEnergy
        """
        return ApparentEnergy(megavoltampere_hours, ApparentEnergyUnits.MegavoltampereHour)

    
    @property
    def voltampere_hours(self) -> float:
        """
        
        """
        if self.__voltampere_hours != None:
            return self.__voltampere_hours
        self.__voltampere_hours = self.__convert_from_base(ApparentEnergyUnits.VoltampereHour)
        return self.__voltampere_hours

    
    @property
    def kilovoltampere_hours(self) -> float:
        """
        
        """
        if self.__kilovoltampere_hours != None:
            return self.__kilovoltampere_hours
        self.__kilovoltampere_hours = self.__convert_from_base(ApparentEnergyUnits.KilovoltampereHour)
        return self.__kilovoltampere_hours

    
    @property
    def megavoltampere_hours(self) -> float:
        """
        
        """
        if self.__megavoltampere_hours != None:
            return self.__megavoltampere_hours
        self.__megavoltampere_hours = self.__convert_from_base(ApparentEnergyUnits.MegavoltampereHour)
        return self.__megavoltampere_hours

    
    def to_string(self, unit: ApparentEnergyUnits = ApparentEnergyUnits.VoltampereHour, fractional_digits: int = None) -> str:
        """
        Format the ApparentEnergy to a string.
        
        Note: the default format for ApparentEnergy is VoltampereHour.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ApparentEnergy. Default is 'VoltampereHour'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ApparentEnergyUnits.VoltampereHour:
            return f"""{super()._truncate_fraction_digits(self.voltampere_hours, fractional_digits)} VAh"""
        
        if unit == ApparentEnergyUnits.KilovoltampereHour:
            return f"""{super()._truncate_fraction_digits(self.kilovoltampere_hours, fractional_digits)} kVAh"""
        
        if unit == ApparentEnergyUnits.MegavoltampereHour:
            return f"""{super()._truncate_fraction_digits(self.megavoltampere_hours, fractional_digits)} MVAh"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ApparentEnergyUnits = ApparentEnergyUnits.VoltampereHour) -> str:
        """
        Get ApparentEnergy unit abbreviation.
        Note! the default abbreviation for ApparentEnergy is VoltampereHour.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ApparentEnergyUnits.VoltampereHour:
            return """VAh"""
        
        if unit_abbreviation == ApparentEnergyUnits.KilovoltampereHour:
            return """kVAh"""
        
        if unit_abbreviation == ApparentEnergyUnits.MegavoltampereHour:
            return """MVAh"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class RadioactivityUnits(Enum):
        """
            RadioactivityUnits enumeration
        """
        
        Becquerel = 'Becquerel'
        """
            Activity of a quantity of radioactive material in which one nucleus decays per second.
        """
        
        Curie = 'Curie'
        """
            
        """
        
        Rutherford = 'Rutherford'
        """
            Activity of a quantity of radioactive material in which one million nuclei decay per second.
        """
        
        Picobecquerel = 'Picobecquerel'
        """
            
        """
        
        Nanobecquerel = 'Nanobecquerel'
        """
            
        """
        
        Microbecquerel = 'Microbecquerel'
        """
            
        """
        
        Millibecquerel = 'Millibecquerel'
        """
            
        """
        
        Kilobecquerel = 'Kilobecquerel'
        """
            
        """
        
        Megabecquerel = 'Megabecquerel'
        """
            
        """
        
        Gigabecquerel = 'Gigabecquerel'
        """
            
        """
        
        Terabecquerel = 'Terabecquerel'
        """
            
        """
        
        Petabecquerel = 'Petabecquerel'
        """
            
        """
        
        Exabecquerel = 'Exabecquerel'
        """
            
        """
        
        Picocurie = 'Picocurie'
        """
            
        """
        
        Nanocurie = 'Nanocurie'
        """
            
        """
        
        Microcurie = 'Microcurie'
        """
            
        """
        
        Millicurie = 'Millicurie'
        """
            
        """
        
        Kilocurie = 'Kilocurie'
        """
            
        """
        
        Megacurie = 'Megacurie'
        """
            
        """
        
        Gigacurie = 'Gigacurie'
        """
            
        """
        
        Teracurie = 'Teracurie'
        """
            
        """
        
        Picorutherford = 'Picorutherford'
        """
            
        """
        
        Nanorutherford = 'Nanorutherford'
        """
            
        """
        
        Microrutherford = 'Microrutherford'
        """
            
        """
        
        Millirutherford = 'Millirutherford'
        """
            
        """
        
        Kilorutherford = 'Kilorutherford'
        """
            
        """
        
        Megarutherford = 'Megarutherford'
        """
            
        """
        
        Gigarutherford = 'Gigarutherford'
        """
            
        """
        
        Terarutherford = 'Terarutherford'
        """
            
        """
        

class RadioactivityDto:
    """
    A DTO representation of a Radioactivity

    Attributes:
        value (float): The value of the Radioactivity.
        unit (RadioactivityUnits): The specific unit that the Radioactivity value is representing.
    """

    def __init__(self, value: float, unit: RadioactivityUnits):
        """
        Create a new DTO representation of a Radioactivity

        Parameters:
            value (float): The value of the Radioactivity.
            unit (RadioactivityUnits): The specific unit that the Radioactivity value is representing.
        """
        self.value: float = value
        """
        The value of the Radioactivity
        """
        self.unit: RadioactivityUnits = unit
        """
        The specific unit that the Radioactivity value is representing
        """

    def to_json(self):
        """
        Get a Radioactivity DTO JSON object representing the current unit.

        :return: JSON object represents Radioactivity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Becquerel"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Radioactivity DTO from a json representation.

        :param data: The Radioactivity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Becquerel"}
        :return: A new instance of RadioactivityDto.
        :rtype: RadioactivityDto
        """
        return RadioactivityDto(value=data["value"], unit=RadioactivityUnits(data["unit"]))


class Radioactivity(AbstractMeasure):
    """
    Amount of ionizing radiation released when an element spontaneously emits energy as a result of the radioactive decay of an unstable atom per unit time.

    Args:
        value (float): The value.
        from_unit (RadioactivityUnits): The Radioactivity unit to create from, The default unit is Becquerel
    """
    def __init__(self, value: float, from_unit: RadioactivityUnits = RadioactivityUnits.Becquerel):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__becquerels = None
        
        self.__curies = None
        
        self.__rutherfords = None
        
        self.__picobecquerels = None
        
        self.__nanobecquerels = None
        
        self.__microbecquerels = None
        
        self.__millibecquerels = None
        
        self.__kilobecquerels = None
        
        self.__megabecquerels = None
        
        self.__gigabecquerels = None
        
        self.__terabecquerels = None
        
        self.__petabecquerels = None
        
        self.__exabecquerels = None
        
        self.__picocuries = None
        
        self.__nanocuries = None
        
        self.__microcuries = None
        
        self.__millicuries = None
        
        self.__kilocuries = None
        
        self.__megacuries = None
        
        self.__gigacuries = None
        
        self.__teracuries = None
        
        self.__picorutherfords = None
        
        self.__nanorutherfords = None
        
        self.__microrutherfords = None
        
        self.__millirutherfords = None
        
        self.__kilorutherfords = None
        
        self.__megarutherfords = None
        
        self.__gigarutherfords = None
        
        self.__terarutherfords = None
        

    def convert(self, unit: RadioactivityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: RadioactivityUnits = RadioactivityUnits.Becquerel) -> RadioactivityDto:
        """
        Get a new instance of Radioactivity DTO representing the current unit.

        :param hold_in_unit: The specific Radioactivity unit to store the Radioactivity value in the DTO representation.
        :type hold_in_unit: RadioactivityUnits
        :return: A new instance of RadioactivityDto.
        :rtype: RadioactivityDto
        """
        return RadioactivityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: RadioactivityUnits = RadioactivityUnits.Becquerel):
        """
        Get a Radioactivity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Radioactivity unit to store the Radioactivity value in the DTO representation.
        :type hold_in_unit: RadioactivityUnits
        :return: JSON object represents Radioactivity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Becquerel"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(radioactivity_dto: RadioactivityDto):
        """
        Obtain a new instance of Radioactivity from a DTO unit object.

        :param radioactivity_dto: The Radioactivity DTO representation.
        :type radioactivity_dto: RadioactivityDto
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(radioactivity_dto.value, radioactivity_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Radioactivity from a DTO unit json representation.

        :param data: The Radioactivity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Becquerel"}
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity.from_dto(RadioactivityDto.from_json(data))

    def __convert_from_base(self, from_unit: RadioactivityUnits) -> float:
        value = self._value
        
        if from_unit == RadioactivityUnits.Becquerel:
            return (value)
        
        if from_unit == RadioactivityUnits.Curie:
            return (value / 3.7e10)
        
        if from_unit == RadioactivityUnits.Rutherford:
            return (value / 1e6)
        
        if from_unit == RadioactivityUnits.Picobecquerel:
            return ((value) / 1e-12)
        
        if from_unit == RadioactivityUnits.Nanobecquerel:
            return ((value) / 1e-09)
        
        if from_unit == RadioactivityUnits.Microbecquerel:
            return ((value) / 1e-06)
        
        if from_unit == RadioactivityUnits.Millibecquerel:
            return ((value) / 0.001)
        
        if from_unit == RadioactivityUnits.Kilobecquerel:
            return ((value) / 1000.0)
        
        if from_unit == RadioactivityUnits.Megabecquerel:
            return ((value) / 1000000.0)
        
        if from_unit == RadioactivityUnits.Gigabecquerel:
            return ((value) / 1000000000.0)
        
        if from_unit == RadioactivityUnits.Terabecquerel:
            return ((value) / 1000000000000.0)
        
        if from_unit == RadioactivityUnits.Petabecquerel:
            return ((value) / 1000000000000000.0)
        
        if from_unit == RadioactivityUnits.Exabecquerel:
            return ((value) / 1e+18)
        
        if from_unit == RadioactivityUnits.Picocurie:
            return ((value / 3.7e10) / 1e-12)
        
        if from_unit == RadioactivityUnits.Nanocurie:
            return ((value / 3.7e10) / 1e-09)
        
        if from_unit == RadioactivityUnits.Microcurie:
            return ((value / 3.7e10) / 1e-06)
        
        if from_unit == RadioactivityUnits.Millicurie:
            return ((value / 3.7e10) / 0.001)
        
        if from_unit == RadioactivityUnits.Kilocurie:
            return ((value / 3.7e10) / 1000.0)
        
        if from_unit == RadioactivityUnits.Megacurie:
            return ((value / 3.7e10) / 1000000.0)
        
        if from_unit == RadioactivityUnits.Gigacurie:
            return ((value / 3.7e10) / 1000000000.0)
        
        if from_unit == RadioactivityUnits.Teracurie:
            return ((value / 3.7e10) / 1000000000000.0)
        
        if from_unit == RadioactivityUnits.Picorutherford:
            return ((value / 1e6) / 1e-12)
        
        if from_unit == RadioactivityUnits.Nanorutherford:
            return ((value / 1e6) / 1e-09)
        
        if from_unit == RadioactivityUnits.Microrutherford:
            return ((value / 1e6) / 1e-06)
        
        if from_unit == RadioactivityUnits.Millirutherford:
            return ((value / 1e6) / 0.001)
        
        if from_unit == RadioactivityUnits.Kilorutherford:
            return ((value / 1e6) / 1000.0)
        
        if from_unit == RadioactivityUnits.Megarutherford:
            return ((value / 1e6) / 1000000.0)
        
        if from_unit == RadioactivityUnits.Gigarutherford:
            return ((value / 1e6) / 1000000000.0)
        
        if from_unit == RadioactivityUnits.Terarutherford:
            return ((value / 1e6) / 1000000000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: RadioactivityUnits) -> float:
        
        if to_unit == RadioactivityUnits.Becquerel:
            return (value)
        
        if to_unit == RadioactivityUnits.Curie:
            return (value * 3.7e10)
        
        if to_unit == RadioactivityUnits.Rutherford:
            return (value * 1e6)
        
        if to_unit == RadioactivityUnits.Picobecquerel:
            return ((value) * 1e-12)
        
        if to_unit == RadioactivityUnits.Nanobecquerel:
            return ((value) * 1e-09)
        
        if to_unit == RadioactivityUnits.Microbecquerel:
            return ((value) * 1e-06)
        
        if to_unit == RadioactivityUnits.Millibecquerel:
            return ((value) * 0.001)
        
        if to_unit == RadioactivityUnits.Kilobecquerel:
            return ((value) * 1000.0)
        
        if to_unit == RadioactivityUnits.Megabecquerel:
            return ((value) * 1000000.0)
        
        if to_unit == RadioactivityUnits.Gigabecquerel:
            return ((value) * 1000000000.0)
        
        if to_unit == RadioactivityUnits.Terabecquerel:
            return ((value) * 1000000000000.0)
        
        if to_unit == RadioactivityUnits.Petabecquerel:
            return ((value) * 1000000000000000.0)
        
        if to_unit == RadioactivityUnits.Exabecquerel:
            return ((value) * 1e+18)
        
        if to_unit == RadioactivityUnits.Picocurie:
            return ((value * 3.7e10) * 1e-12)
        
        if to_unit == RadioactivityUnits.Nanocurie:
            return ((value * 3.7e10) * 1e-09)
        
        if to_unit == RadioactivityUnits.Microcurie:
            return ((value * 3.7e10) * 1e-06)
        
        if to_unit == RadioactivityUnits.Millicurie:
            return ((value * 3.7e10) * 0.001)
        
        if to_unit == RadioactivityUnits.Kilocurie:
            return ((value * 3.7e10) * 1000.0)
        
        if to_unit == RadioactivityUnits.Megacurie:
            return ((value * 3.7e10) * 1000000.0)
        
        if to_unit == RadioactivityUnits.Gigacurie:
            return ((value * 3.7e10) * 1000000000.0)
        
        if to_unit == RadioactivityUnits.Teracurie:
            return ((value * 3.7e10) * 1000000000000.0)
        
        if to_unit == RadioactivityUnits.Picorutherford:
            return ((value * 1e6) * 1e-12)
        
        if to_unit == RadioactivityUnits.Nanorutherford:
            return ((value * 1e6) * 1e-09)
        
        if to_unit == RadioactivityUnits.Microrutherford:
            return ((value * 1e6) * 1e-06)
        
        if to_unit == RadioactivityUnits.Millirutherford:
            return ((value * 1e6) * 0.001)
        
        if to_unit == RadioactivityUnits.Kilorutherford:
            return ((value * 1e6) * 1000.0)
        
        if to_unit == RadioactivityUnits.Megarutherford:
            return ((value * 1e6) * 1000000.0)
        
        if to_unit == RadioactivityUnits.Gigarutherford:
            return ((value * 1e6) * 1000000000.0)
        
        if to_unit == RadioactivityUnits.Terarutherford:
            return ((value * 1e6) * 1000000000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_becquerels(becquerels: float):
        """
        Create a new instance of Radioactivity from a value in becquerels.

        Activity of a quantity of radioactive material in which one nucleus decays per second.

        :param meters: The Radioactivity value in becquerels.
        :type becquerels: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(becquerels, RadioactivityUnits.Becquerel)

    
    @staticmethod
    def from_curies(curies: float):
        """
        Create a new instance of Radioactivity from a value in curies.

        

        :param meters: The Radioactivity value in curies.
        :type curies: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(curies, RadioactivityUnits.Curie)

    
    @staticmethod
    def from_rutherfords(rutherfords: float):
        """
        Create a new instance of Radioactivity from a value in rutherfords.

        Activity of a quantity of radioactive material in which one million nuclei decay per second.

        :param meters: The Radioactivity value in rutherfords.
        :type rutherfords: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(rutherfords, RadioactivityUnits.Rutherford)

    
    @staticmethod
    def from_picobecquerels(picobecquerels: float):
        """
        Create a new instance of Radioactivity from a value in picobecquerels.

        

        :param meters: The Radioactivity value in picobecquerels.
        :type picobecquerels: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(picobecquerels, RadioactivityUnits.Picobecquerel)

    
    @staticmethod
    def from_nanobecquerels(nanobecquerels: float):
        """
        Create a new instance of Radioactivity from a value in nanobecquerels.

        

        :param meters: The Radioactivity value in nanobecquerels.
        :type nanobecquerels: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(nanobecquerels, RadioactivityUnits.Nanobecquerel)

    
    @staticmethod
    def from_microbecquerels(microbecquerels: float):
        """
        Create a new instance of Radioactivity from a value in microbecquerels.

        

        :param meters: The Radioactivity value in microbecquerels.
        :type microbecquerels: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(microbecquerels, RadioactivityUnits.Microbecquerel)

    
    @staticmethod
    def from_millibecquerels(millibecquerels: float):
        """
        Create a new instance of Radioactivity from a value in millibecquerels.

        

        :param meters: The Radioactivity value in millibecquerels.
        :type millibecquerels: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(millibecquerels, RadioactivityUnits.Millibecquerel)

    
    @staticmethod
    def from_kilobecquerels(kilobecquerels: float):
        """
        Create a new instance of Radioactivity from a value in kilobecquerels.

        

        :param meters: The Radioactivity value in kilobecquerels.
        :type kilobecquerels: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(kilobecquerels, RadioactivityUnits.Kilobecquerel)

    
    @staticmethod
    def from_megabecquerels(megabecquerels: float):
        """
        Create a new instance of Radioactivity from a value in megabecquerels.

        

        :param meters: The Radioactivity value in megabecquerels.
        :type megabecquerels: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(megabecquerels, RadioactivityUnits.Megabecquerel)

    
    @staticmethod
    def from_gigabecquerels(gigabecquerels: float):
        """
        Create a new instance of Radioactivity from a value in gigabecquerels.

        

        :param meters: The Radioactivity value in gigabecquerels.
        :type gigabecquerels: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(gigabecquerels, RadioactivityUnits.Gigabecquerel)

    
    @staticmethod
    def from_terabecquerels(terabecquerels: float):
        """
        Create a new instance of Radioactivity from a value in terabecquerels.

        

        :param meters: The Radioactivity value in terabecquerels.
        :type terabecquerels: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(terabecquerels, RadioactivityUnits.Terabecquerel)

    
    @staticmethod
    def from_petabecquerels(petabecquerels: float):
        """
        Create a new instance of Radioactivity from a value in petabecquerels.

        

        :param meters: The Radioactivity value in petabecquerels.
        :type petabecquerels: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(petabecquerels, RadioactivityUnits.Petabecquerel)

    
    @staticmethod
    def from_exabecquerels(exabecquerels: float):
        """
        Create a new instance of Radioactivity from a value in exabecquerels.

        

        :param meters: The Radioactivity value in exabecquerels.
        :type exabecquerels: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(exabecquerels, RadioactivityUnits.Exabecquerel)

    
    @staticmethod
    def from_picocuries(picocuries: float):
        """
        Create a new instance of Radioactivity from a value in picocuries.

        

        :param meters: The Radioactivity value in picocuries.
        :type picocuries: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(picocuries, RadioactivityUnits.Picocurie)

    
    @staticmethod
    def from_nanocuries(nanocuries: float):
        """
        Create a new instance of Radioactivity from a value in nanocuries.

        

        :param meters: The Radioactivity value in nanocuries.
        :type nanocuries: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(nanocuries, RadioactivityUnits.Nanocurie)

    
    @staticmethod
    def from_microcuries(microcuries: float):
        """
        Create a new instance of Radioactivity from a value in microcuries.

        

        :param meters: The Radioactivity value in microcuries.
        :type microcuries: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(microcuries, RadioactivityUnits.Microcurie)

    
    @staticmethod
    def from_millicuries(millicuries: float):
        """
        Create a new instance of Radioactivity from a value in millicuries.

        

        :param meters: The Radioactivity value in millicuries.
        :type millicuries: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(millicuries, RadioactivityUnits.Millicurie)

    
    @staticmethod
    def from_kilocuries(kilocuries: float):
        """
        Create a new instance of Radioactivity from a value in kilocuries.

        

        :param meters: The Radioactivity value in kilocuries.
        :type kilocuries: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(kilocuries, RadioactivityUnits.Kilocurie)

    
    @staticmethod
    def from_megacuries(megacuries: float):
        """
        Create a new instance of Radioactivity from a value in megacuries.

        

        :param meters: The Radioactivity value in megacuries.
        :type megacuries: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(megacuries, RadioactivityUnits.Megacurie)

    
    @staticmethod
    def from_gigacuries(gigacuries: float):
        """
        Create a new instance of Radioactivity from a value in gigacuries.

        

        :param meters: The Radioactivity value in gigacuries.
        :type gigacuries: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(gigacuries, RadioactivityUnits.Gigacurie)

    
    @staticmethod
    def from_teracuries(teracuries: float):
        """
        Create a new instance of Radioactivity from a value in teracuries.

        

        :param meters: The Radioactivity value in teracuries.
        :type teracuries: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(teracuries, RadioactivityUnits.Teracurie)

    
    @staticmethod
    def from_picorutherfords(picorutherfords: float):
        """
        Create a new instance of Radioactivity from a value in picorutherfords.

        

        :param meters: The Radioactivity value in picorutherfords.
        :type picorutherfords: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(picorutherfords, RadioactivityUnits.Picorutherford)

    
    @staticmethod
    def from_nanorutherfords(nanorutherfords: float):
        """
        Create a new instance of Radioactivity from a value in nanorutherfords.

        

        :param meters: The Radioactivity value in nanorutherfords.
        :type nanorutherfords: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(nanorutherfords, RadioactivityUnits.Nanorutherford)

    
    @staticmethod
    def from_microrutherfords(microrutherfords: float):
        """
        Create a new instance of Radioactivity from a value in microrutherfords.

        

        :param meters: The Radioactivity value in microrutherfords.
        :type microrutherfords: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(microrutherfords, RadioactivityUnits.Microrutherford)

    
    @staticmethod
    def from_millirutherfords(millirutherfords: float):
        """
        Create a new instance of Radioactivity from a value in millirutherfords.

        

        :param meters: The Radioactivity value in millirutherfords.
        :type millirutherfords: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(millirutherfords, RadioactivityUnits.Millirutherford)

    
    @staticmethod
    def from_kilorutherfords(kilorutherfords: float):
        """
        Create a new instance of Radioactivity from a value in kilorutherfords.

        

        :param meters: The Radioactivity value in kilorutherfords.
        :type kilorutherfords: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(kilorutherfords, RadioactivityUnits.Kilorutherford)

    
    @staticmethod
    def from_megarutherfords(megarutherfords: float):
        """
        Create a new instance of Radioactivity from a value in megarutherfords.

        

        :param meters: The Radioactivity value in megarutherfords.
        :type megarutherfords: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(megarutherfords, RadioactivityUnits.Megarutherford)

    
    @staticmethod
    def from_gigarutherfords(gigarutherfords: float):
        """
        Create a new instance of Radioactivity from a value in gigarutherfords.

        

        :param meters: The Radioactivity value in gigarutherfords.
        :type gigarutherfords: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(gigarutherfords, RadioactivityUnits.Gigarutherford)

    
    @staticmethod
    def from_terarutherfords(terarutherfords: float):
        """
        Create a new instance of Radioactivity from a value in terarutherfords.

        

        :param meters: The Radioactivity value in terarutherfords.
        :type terarutherfords: float
        :return: A new instance of Radioactivity.
        :rtype: Radioactivity
        """
        return Radioactivity(terarutherfords, RadioactivityUnits.Terarutherford)

    
    @property
    def becquerels(self) -> float:
        """
        Activity of a quantity of radioactive material in which one nucleus decays per second.
        """
        if self.__becquerels != None:
            return self.__becquerels
        self.__becquerels = self.__convert_from_base(RadioactivityUnits.Becquerel)
        return self.__becquerels

    
    @property
    def curies(self) -> float:
        """
        
        """
        if self.__curies != None:
            return self.__curies
        self.__curies = self.__convert_from_base(RadioactivityUnits.Curie)
        return self.__curies

    
    @property
    def rutherfords(self) -> float:
        """
        Activity of a quantity of radioactive material in which one million nuclei decay per second.
        """
        if self.__rutherfords != None:
            return self.__rutherfords
        self.__rutherfords = self.__convert_from_base(RadioactivityUnits.Rutherford)
        return self.__rutherfords

    
    @property
    def picobecquerels(self) -> float:
        """
        
        """
        if self.__picobecquerels != None:
            return self.__picobecquerels
        self.__picobecquerels = self.__convert_from_base(RadioactivityUnits.Picobecquerel)
        return self.__picobecquerels

    
    @property
    def nanobecquerels(self) -> float:
        """
        
        """
        if self.__nanobecquerels != None:
            return self.__nanobecquerels
        self.__nanobecquerels = self.__convert_from_base(RadioactivityUnits.Nanobecquerel)
        return self.__nanobecquerels

    
    @property
    def microbecquerels(self) -> float:
        """
        
        """
        if self.__microbecquerels != None:
            return self.__microbecquerels
        self.__microbecquerels = self.__convert_from_base(RadioactivityUnits.Microbecquerel)
        return self.__microbecquerels

    
    @property
    def millibecquerels(self) -> float:
        """
        
        """
        if self.__millibecquerels != None:
            return self.__millibecquerels
        self.__millibecquerels = self.__convert_from_base(RadioactivityUnits.Millibecquerel)
        return self.__millibecquerels

    
    @property
    def kilobecquerels(self) -> float:
        """
        
        """
        if self.__kilobecquerels != None:
            return self.__kilobecquerels
        self.__kilobecquerels = self.__convert_from_base(RadioactivityUnits.Kilobecquerel)
        return self.__kilobecquerels

    
    @property
    def megabecquerels(self) -> float:
        """
        
        """
        if self.__megabecquerels != None:
            return self.__megabecquerels
        self.__megabecquerels = self.__convert_from_base(RadioactivityUnits.Megabecquerel)
        return self.__megabecquerels

    
    @property
    def gigabecquerels(self) -> float:
        """
        
        """
        if self.__gigabecquerels != None:
            return self.__gigabecquerels
        self.__gigabecquerels = self.__convert_from_base(RadioactivityUnits.Gigabecquerel)
        return self.__gigabecquerels

    
    @property
    def terabecquerels(self) -> float:
        """
        
        """
        if self.__terabecquerels != None:
            return self.__terabecquerels
        self.__terabecquerels = self.__convert_from_base(RadioactivityUnits.Terabecquerel)
        return self.__terabecquerels

    
    @property
    def petabecquerels(self) -> float:
        """
        
        """
        if self.__petabecquerels != None:
            return self.__petabecquerels
        self.__petabecquerels = self.__convert_from_base(RadioactivityUnits.Petabecquerel)
        return self.__petabecquerels

    
    @property
    def exabecquerels(self) -> float:
        """
        
        """
        if self.__exabecquerels != None:
            return self.__exabecquerels
        self.__exabecquerels = self.__convert_from_base(RadioactivityUnits.Exabecquerel)
        return self.__exabecquerels

    
    @property
    def picocuries(self) -> float:
        """
        
        """
        if self.__picocuries != None:
            return self.__picocuries
        self.__picocuries = self.__convert_from_base(RadioactivityUnits.Picocurie)
        return self.__picocuries

    
    @property
    def nanocuries(self) -> float:
        """
        
        """
        if self.__nanocuries != None:
            return self.__nanocuries
        self.__nanocuries = self.__convert_from_base(RadioactivityUnits.Nanocurie)
        return self.__nanocuries

    
    @property
    def microcuries(self) -> float:
        """
        
        """
        if self.__microcuries != None:
            return self.__microcuries
        self.__microcuries = self.__convert_from_base(RadioactivityUnits.Microcurie)
        return self.__microcuries

    
    @property
    def millicuries(self) -> float:
        """
        
        """
        if self.__millicuries != None:
            return self.__millicuries
        self.__millicuries = self.__convert_from_base(RadioactivityUnits.Millicurie)
        return self.__millicuries

    
    @property
    def kilocuries(self) -> float:
        """
        
        """
        if self.__kilocuries != None:
            return self.__kilocuries
        self.__kilocuries = self.__convert_from_base(RadioactivityUnits.Kilocurie)
        return self.__kilocuries

    
    @property
    def megacuries(self) -> float:
        """
        
        """
        if self.__megacuries != None:
            return self.__megacuries
        self.__megacuries = self.__convert_from_base(RadioactivityUnits.Megacurie)
        return self.__megacuries

    
    @property
    def gigacuries(self) -> float:
        """
        
        """
        if self.__gigacuries != None:
            return self.__gigacuries
        self.__gigacuries = self.__convert_from_base(RadioactivityUnits.Gigacurie)
        return self.__gigacuries

    
    @property
    def teracuries(self) -> float:
        """
        
        """
        if self.__teracuries != None:
            return self.__teracuries
        self.__teracuries = self.__convert_from_base(RadioactivityUnits.Teracurie)
        return self.__teracuries

    
    @property
    def picorutherfords(self) -> float:
        """
        
        """
        if self.__picorutherfords != None:
            return self.__picorutherfords
        self.__picorutherfords = self.__convert_from_base(RadioactivityUnits.Picorutherford)
        return self.__picorutherfords

    
    @property
    def nanorutherfords(self) -> float:
        """
        
        """
        if self.__nanorutherfords != None:
            return self.__nanorutherfords
        self.__nanorutherfords = self.__convert_from_base(RadioactivityUnits.Nanorutherford)
        return self.__nanorutherfords

    
    @property
    def microrutherfords(self) -> float:
        """
        
        """
        if self.__microrutherfords != None:
            return self.__microrutherfords
        self.__microrutherfords = self.__convert_from_base(RadioactivityUnits.Microrutherford)
        return self.__microrutherfords

    
    @property
    def millirutherfords(self) -> float:
        """
        
        """
        if self.__millirutherfords != None:
            return self.__millirutherfords
        self.__millirutherfords = self.__convert_from_base(RadioactivityUnits.Millirutherford)
        return self.__millirutherfords

    
    @property
    def kilorutherfords(self) -> float:
        """
        
        """
        if self.__kilorutherfords != None:
            return self.__kilorutherfords
        self.__kilorutherfords = self.__convert_from_base(RadioactivityUnits.Kilorutherford)
        return self.__kilorutherfords

    
    @property
    def megarutherfords(self) -> float:
        """
        
        """
        if self.__megarutherfords != None:
            return self.__megarutherfords
        self.__megarutherfords = self.__convert_from_base(RadioactivityUnits.Megarutherford)
        return self.__megarutherfords

    
    @property
    def gigarutherfords(self) -> float:
        """
        
        """
        if self.__gigarutherfords != None:
            return self.__gigarutherfords
        self.__gigarutherfords = self.__convert_from_base(RadioactivityUnits.Gigarutherford)
        return self.__gigarutherfords

    
    @property
    def terarutherfords(self) -> float:
        """
        
        """
        if self.__terarutherfords != None:
            return self.__terarutherfords
        self.__terarutherfords = self.__convert_from_base(RadioactivityUnits.Terarutherford)
        return self.__terarutherfords

    
    def to_string(self, unit: RadioactivityUnits = RadioactivityUnits.Becquerel, fractional_digits: int = None) -> str:
        """
        Format the Radioactivity to a string.
        
        Note: the default format for Radioactivity is Becquerel.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Radioactivity. Default is 'Becquerel'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == RadioactivityUnits.Becquerel:
            return f"""{super()._truncate_fraction_digits(self.becquerels, fractional_digits)} Bq"""
        
        if unit == RadioactivityUnits.Curie:
            return f"""{super()._truncate_fraction_digits(self.curies, fractional_digits)} Ci"""
        
        if unit == RadioactivityUnits.Rutherford:
            return f"""{super()._truncate_fraction_digits(self.rutherfords, fractional_digits)} Rd"""
        
        if unit == RadioactivityUnits.Picobecquerel:
            return f"""{super()._truncate_fraction_digits(self.picobecquerels, fractional_digits)} pBq"""
        
        if unit == RadioactivityUnits.Nanobecquerel:
            return f"""{super()._truncate_fraction_digits(self.nanobecquerels, fractional_digits)} nBq"""
        
        if unit == RadioactivityUnits.Microbecquerel:
            return f"""{super()._truncate_fraction_digits(self.microbecquerels, fractional_digits)} μBq"""
        
        if unit == RadioactivityUnits.Millibecquerel:
            return f"""{super()._truncate_fraction_digits(self.millibecquerels, fractional_digits)} mBq"""
        
        if unit == RadioactivityUnits.Kilobecquerel:
            return f"""{super()._truncate_fraction_digits(self.kilobecquerels, fractional_digits)} kBq"""
        
        if unit == RadioactivityUnits.Megabecquerel:
            return f"""{super()._truncate_fraction_digits(self.megabecquerels, fractional_digits)} MBq"""
        
        if unit == RadioactivityUnits.Gigabecquerel:
            return f"""{super()._truncate_fraction_digits(self.gigabecquerels, fractional_digits)} GBq"""
        
        if unit == RadioactivityUnits.Terabecquerel:
            return f"""{super()._truncate_fraction_digits(self.terabecquerels, fractional_digits)} TBq"""
        
        if unit == RadioactivityUnits.Petabecquerel:
            return f"""{super()._truncate_fraction_digits(self.petabecquerels, fractional_digits)} PBq"""
        
        if unit == RadioactivityUnits.Exabecquerel:
            return f"""{super()._truncate_fraction_digits(self.exabecquerels, fractional_digits)} EBq"""
        
        if unit == RadioactivityUnits.Picocurie:
            return f"""{super()._truncate_fraction_digits(self.picocuries, fractional_digits)} pCi"""
        
        if unit == RadioactivityUnits.Nanocurie:
            return f"""{super()._truncate_fraction_digits(self.nanocuries, fractional_digits)} nCi"""
        
        if unit == RadioactivityUnits.Microcurie:
            return f"""{super()._truncate_fraction_digits(self.microcuries, fractional_digits)} μCi"""
        
        if unit == RadioactivityUnits.Millicurie:
            return f"""{super()._truncate_fraction_digits(self.millicuries, fractional_digits)} mCi"""
        
        if unit == RadioactivityUnits.Kilocurie:
            return f"""{super()._truncate_fraction_digits(self.kilocuries, fractional_digits)} kCi"""
        
        if unit == RadioactivityUnits.Megacurie:
            return f"""{super()._truncate_fraction_digits(self.megacuries, fractional_digits)} MCi"""
        
        if unit == RadioactivityUnits.Gigacurie:
            return f"""{super()._truncate_fraction_digits(self.gigacuries, fractional_digits)} GCi"""
        
        if unit == RadioactivityUnits.Teracurie:
            return f"""{super()._truncate_fraction_digits(self.teracuries, fractional_digits)} TCi"""
        
        if unit == RadioactivityUnits.Picorutherford:
            return f"""{super()._truncate_fraction_digits(self.picorutherfords, fractional_digits)} pRd"""
        
        if unit == RadioactivityUnits.Nanorutherford:
            return f"""{super()._truncate_fraction_digits(self.nanorutherfords, fractional_digits)} nRd"""
        
        if unit == RadioactivityUnits.Microrutherford:
            return f"""{super()._truncate_fraction_digits(self.microrutherfords, fractional_digits)} μRd"""
        
        if unit == RadioactivityUnits.Millirutherford:
            return f"""{super()._truncate_fraction_digits(self.millirutherfords, fractional_digits)} mRd"""
        
        if unit == RadioactivityUnits.Kilorutherford:
            return f"""{super()._truncate_fraction_digits(self.kilorutherfords, fractional_digits)} kRd"""
        
        if unit == RadioactivityUnits.Megarutherford:
            return f"""{super()._truncate_fraction_digits(self.megarutherfords, fractional_digits)} MRd"""
        
        if unit == RadioactivityUnits.Gigarutherford:
            return f"""{super()._truncate_fraction_digits(self.gigarutherfords, fractional_digits)} GRd"""
        
        if unit == RadioactivityUnits.Terarutherford:
            return f"""{super()._truncate_fraction_digits(self.terarutherfords, fractional_digits)} TRd"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: RadioactivityUnits = RadioactivityUnits.Becquerel) -> str:
        """
        Get Radioactivity unit abbreviation.
        Note! the default abbreviation for Radioactivity is Becquerel.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == RadioactivityUnits.Becquerel:
            return """Bq"""
        
        if unit_abbreviation == RadioactivityUnits.Curie:
            return """Ci"""
        
        if unit_abbreviation == RadioactivityUnits.Rutherford:
            return """Rd"""
        
        if unit_abbreviation == RadioactivityUnits.Picobecquerel:
            return """pBq"""
        
        if unit_abbreviation == RadioactivityUnits.Nanobecquerel:
            return """nBq"""
        
        if unit_abbreviation == RadioactivityUnits.Microbecquerel:
            return """μBq"""
        
        if unit_abbreviation == RadioactivityUnits.Millibecquerel:
            return """mBq"""
        
        if unit_abbreviation == RadioactivityUnits.Kilobecquerel:
            return """kBq"""
        
        if unit_abbreviation == RadioactivityUnits.Megabecquerel:
            return """MBq"""
        
        if unit_abbreviation == RadioactivityUnits.Gigabecquerel:
            return """GBq"""
        
        if unit_abbreviation == RadioactivityUnits.Terabecquerel:
            return """TBq"""
        
        if unit_abbreviation == RadioactivityUnits.Petabecquerel:
            return """PBq"""
        
        if unit_abbreviation == RadioactivityUnits.Exabecquerel:
            return """EBq"""
        
        if unit_abbreviation == RadioactivityUnits.Picocurie:
            return """pCi"""
        
        if unit_abbreviation == RadioactivityUnits.Nanocurie:
            return """nCi"""
        
        if unit_abbreviation == RadioactivityUnits.Microcurie:
            return """μCi"""
        
        if unit_abbreviation == RadioactivityUnits.Millicurie:
            return """mCi"""
        
        if unit_abbreviation == RadioactivityUnits.Kilocurie:
            return """kCi"""
        
        if unit_abbreviation == RadioactivityUnits.Megacurie:
            return """MCi"""
        
        if unit_abbreviation == RadioactivityUnits.Gigacurie:
            return """GCi"""
        
        if unit_abbreviation == RadioactivityUnits.Teracurie:
            return """TCi"""
        
        if unit_abbreviation == RadioactivityUnits.Picorutherford:
            return """pRd"""
        
        if unit_abbreviation == RadioactivityUnits.Nanorutherford:
            return """nRd"""
        
        if unit_abbreviation == RadioactivityUnits.Microrutherford:
            return """μRd"""
        
        if unit_abbreviation == RadioactivityUnits.Millirutherford:
            return """mRd"""
        
        if unit_abbreviation == RadioactivityUnits.Kilorutherford:
            return """kRd"""
        
        if unit_abbreviation == RadioactivityUnits.Megarutherford:
            return """MRd"""
        
        if unit_abbreviation == RadioactivityUnits.Gigarutherford:
            return """GRd"""
        
        if unit_abbreviation == RadioactivityUnits.Terarutherford:
            return """TRd"""
        